@@ -385,8 +385,7 @@ void move_everyone(void)
 
    /* fire bullet? */
    if (!shot) {
-      if ((key[KEY_SPACE] || key[KEY_LCONTROL] || key[KEY_RCONTROL]) ||
-	 (joy[0].button[0].b) || (joy[0].button[1].b)) {
+      if ((key[KEY_SPACE]) || (joy[0].button[0].b) || (joy[0].button[1].b)) {
 	 if (prev_bullet_time + BULLET_DELAY < game_time) {
 	    bullet = add_bullet((pos >> SPEED_SHIFT) - 2, SCREEN_H - 64);
 	    if (bullet) {
@@ -657,9 +656,9 @@ void draw_screen(void)
 
    /* draw the score and fps information */
    if (fps)
-      sprintf(score_buf, "Lives: %d - Score: %d - (%s, %d fps)", ship_count, score, animation_type_str, fps);
+      sprintf(score_buf, "Lives: %d - Score: %ld - (%s, %ld fps)", ship_count, (long)score, animation_type_str, (long)fps);
    else
-      sprintf(score_buf, "Lives: %d - Score: %d", ship_count, score);
+      sprintf(score_buf, "Lives: %d - Score: %ld", ship_count, (long)score);
 
    textout_ex(bmp, font, score_buf, 0, 0, 7, 0);
 
@@ -930,7 +929,6 @@ void play_game(void)
    b2 = create_bitmap(160, 160);
    clear_bitmap(b);
    textout_centre_ex(b, data[END_FONT].dat, "GAME OVER", 80, 50, 2, 0);
-   sprintf(score_buf, "Score: %d", score);
    textout_centre_ex(b, data[END_FONT].dat, score_buf, 80, 82, 2, 0);
    clear_keybuf();
    scroll_count = -160;
@@ -2068,7 +2066,7 @@ int title_screen(void)
 
 int main(int argc, char *argv[])
 {
-   int c, w, h, num_pages;
+   int c, w, h;
    char buf[256], buf2[256];
    BITMAP *bmp;
    CREDIT_NAME *cred;
@@ -2164,8 +2162,12 @@ int main(int argc, char *argv[])
       destroy_bitmap(bmp);
    }
 
+   c = retrace_count;
+
    if (!jumpstart) {
-      rest (1000);
+      do {
+      } while (retrace_count-c < 120);
+
       fade_out(1);
    }
 
@@ -2178,29 +2180,9 @@ int main(int argc, char *argv[])
 
    if (pick_animation_type(&animation_type) < 0)
       exit(1);
-      
-   switch (animation_type) {
-
-      case PAGE_FLIP:
-      case RETRACE_FLIP: 
-	 num_pages = 2;
-	 break;
-
-      case TRIPLE_BUFFER: 
-	 num_pages = 3;
-	 break;
-
-      default: 
-	 num_pages = 1;
-	 break;
-   }
 
    set_color_depth(8);
-#ifdef ALLEGRO_VRAM_SINGLE_SURFACE
-   if (set_gfx_mode(c, w, h, w, h * num_pages) != 0) {
-#else
    if (set_gfx_mode(c, w, h, 0, 0) != 0) {
-#endif
       set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
       allegro_message("Error setting 8bpp graphics mode\n%s\n", allegro_error);
       exit(1);
