@@ -3,10 +3,6 @@
  *
  *    This program demonstrates how to use the get_camera_matrix() function
  *    to view a 3d world from any position and angle.
- *
- *    Modified by Francisco Pires:
- *    + Added FPS counter.
- *    + Added option to enable/disable waiting for vsync.
  */
 
 
@@ -29,14 +25,6 @@
 #define DEG(n)    ((n) * 180.0 / M_PI)
 
 
-/* how many times per second the fps will be checked */
-#define FPS_INT 2
-
-
-/* uncomment to disable waiting for vsync */
-/* #define DISABLE_VSYNC */
-
-
 /* parameters controlling the camera and projection state */
 int viewport_w = 320;
 int viewport_h = 240;
@@ -49,9 +37,6 @@ float heading = 0;
 float pitch = 0;
 float roll = 0;
 
-int fps;
-int framecount;
-
 
 
 /* render a tile of the grid */
@@ -161,7 +146,7 @@ void render(BITMAP *bmp)
 
    set_projection_viewport(x, y, w, h);
    rect(bmp, x-1, y-1, x+w, y+h, makecol(255, 0, 0));
-   set_clip_rect(bmp, x, y, x+w-1, y+h-1);
+   set_clip(bmp, x, y, x+w-1, y+h-1);
 
    /* calculate the in-front vector */
    xfront = sin(heading) * cos(pitch);
@@ -186,7 +171,7 @@ void render(BITMAP *bmp)
 	 draw_square(bmp, &camera, x, y);
 
    /* overlay some text */
-   set_clip_rect(bmp, 0, 0, bmp->w, bmp->h);
+   set_clip(bmp, 0, 0, bmp->w, bmp->h);
    textprintf_ex(bmp, font, 0,  0, makecol(0, 0, 0), -1, "Viewport width: %d (w/W changes)", viewport_w);
    textprintf_ex(bmp, font, 0,  8, makecol(0, 0, 0), -1, "Viewport height: %d (h/H changes)", viewport_h);
    textprintf_ex(bmp, font, 0, 16, makecol(0, 0, 0), -1, "Field of view: %d (f/F changes)", fov);
@@ -199,7 +184,6 @@ void render(BITMAP *bmp)
    textprintf_ex(bmp, font, 0, 72, makecol(0, 0, 0), -1, "Roll: %.2f deg (r/R changes)", DEG(roll));
    textprintf_ex(bmp, font, 0, 80, makecol(0, 0, 0), -1, "Front vector: %.2f, %.2f, %.2f", xfront, yfront, zfront);
    textprintf_ex(bmp, font, 0, 88, makecol(0, 0, 0), -1, "Up vector: %.2f, %.2f, %.2f", xup, yup, zup);
-   textprintf_ex(bmp, font, 0, 96, makecol(0, 0, 0), -1, "Frames per second: %d", fps);
 }
 
 
@@ -324,14 +308,6 @@ void process_input(void)
 
 
 
-void fps_check(void)
-{
-   fps = framecount*FPS_INT;
-   framecount = 0;
-}
-
-
-
 int main(void)
 {
    BITMAP *buffer;
@@ -339,7 +315,6 @@ int main(void)
    if (allegro_init() != 0)
       return 1;
    install_keyboard();
-   install_timer();
 
    if (set_gfx_mode(GFX_AUTODETECT, 640, 480, 0, 0) != 0) {
       if (set_gfx_mode(GFX_SAFE, 640, 480, 0, 0) != 0) {
@@ -351,17 +326,12 @@ int main(void)
 
    set_palette(desktop_palette);
    buffer = create_bitmap(SCREEN_W, SCREEN_H);
-   install_int_ex(fps_check, BPS_TO_TIMER(FPS_INT));
 
    while (!key[KEY_ESC]) {
       render(buffer);
 
-      #ifndef DISABLE_VSYNC
-         vsync();
-      #endif
-      
-      blit(buffer, screen, 0, 0, 0, 0, SCREEN_W, SCREEN_H);       
-      framecount++;
+      vsync();
+      blit(buffer, screen, 0, 0, 0, 0, SCREEN_W, SCREEN_H); 
 
       process_input();
    }
