@@ -169,11 +169,12 @@ int main(int argc, char *argv[])
 		      "    + avoids the need for a memory to screen blit of the completed image\n"
 		      "    + can use hardware acceleration when it is available\n"
 		      "    - drawing directly to vram can be slower than using a memory bitmap\n"
-		      "    - requires a card with enough video memory for two screen pages\n\n"
+		      "    - requires a card with enough video memory for two screen pages\n"
+		      "    - some VESA drivers have nasty bugs in the scrolling functions\n\n"
 		      "3 = triple buffered (three pages of video memory)\n\n"
 		      "    + like page flipping, but faster and smoother\n"
 		      "    - requires a card with enough video memory for three screen pages\n"
-		      "    - only possible with some graphics drivers\n\n"
+		      "    - only possible in mode-X (not win95), or with VBE 3.0 or VBE/AF\n\n"
 		      "4 = double buffered (system bitmap)\n\n"
 		      "    + as easy as normal double buffering\n"
 		      "    + system bitmaps can be hardware accelerated on some platforms\n"
@@ -200,11 +201,7 @@ int main(int argc, char *argv[])
 
    set_color_depth(bpp);
 
-#ifdef ALLEGRO_VRAM_SINGLE_SURFACE
-   if (set_gfx_mode(card, w, h, w, num_pages*h) != 0) {
-#else
    if (set_gfx_mode(card, w, h, 0, 0) != 0) {
-#endif
       set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
       allegro_message("Error setting graphics mode\n%s\n", allegro_error);
       return 1;
