@@ -36,20 +36,6 @@
 AL_VAR(int, _allegro_count);
 
 
-/* flag to know whether we are being called by the exit mechanism */
-AL_VAR(int, _allegro_in_exit);
-
-
-/* flag to decide whether to disable the screensaver */
-enum {
-  NEVER_DISABLED,
-  FULLSCREEN_DISABLED,
-  ALWAYS_DISABLED
-};
-
-AL_VAR(int, _screensaver_policy);
-
-
 /* some Allegro functions need a block of scratch memory */
 AL_VAR(void *, _scratch_mem);
 AL_VAR(int, _scratch_mem_size);
@@ -461,7 +447,6 @@ AL_FUNC(unsigned long, _blender_write_alpha, (unsigned long x, unsigned long y,
 
 /* graphics drawing routines */
 AL_FUNC(void, _normal_line, (BITMAP *bmp, int x1, int y1, int x2, int y2, int color));
-AL_FUNC(void, _fast_line, (BITMAP *bmp, int x1, int y1, int x2, int y2, int color));
 AL_FUNC(void, _normal_rectfill, (BITMAP *bmp, int x1, int y1, int x2, int y2, int color));
 
 #ifdef ALLEGRO_COLOR8
@@ -1176,8 +1161,6 @@ AL_FUNC(void, _driver_list_append_list, (_DRIVER_INFO **drvlist, _DRIVER_INFO *s
 
 /* various libc stuff */
 AL_FUNC(void *, _al_sane_realloc, (void *ptr, size_t size));
-AL_FUNC(char *, _al_sane_strncpy, (char *dest, const char *src, size_t n));
-
 
 #define _AL_RAND_MAX  0xFFFF
 AL_FUNC(void, _al_srand, (int seed));
