@@ -10,19 +10,23 @@
  *
  *      The keyboard configuration utility program.
  *
- *      By Shawn Hargreaves.
+ *      By Shawn Hargreaves, modified by Peter Pavlovic.
  *
  *      See readme.txt for copyright information.
  */
 
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 
 #include "allegro.h"
-#include "allegro/aintern.h"
+#include "allegro/internal/aintern.h"
 
 
+/* The code can't link on platforms that don't use src/misc/pckeys.c (Linux console, MacOS X). */
+#if (defined ALLEGRO_DOS) || (defined ALLEGRO_WINDOWS) || (defined ALLEGRO_WITH_XWINDOWS) || (defined ALLEGRO_QNX) || (defined ALLEGRO_BEOS)
+
 
 char *scancode_name[] = 
 {
@@ -49,7 +53,10 @@ char *scancode_name[] =
    "KEY_PGUP",       "KEY_PGDN",       "KEY_LEFT",       "KEY_RIGHT",
    "KEY_UP",         "KEY_DOWN",       "KEY_SLASH_PAD",  "KEY_ASTERISK",
    "KEY_MINUS_PAD",  "KEY_PLUS_PAD",   "KEY_DEL_PAD",    "KEY_ENTER_PAD",
-   "KEY_PRTSCR",     "KEY_PAUSE",      "KEY_YEN",        "KEY_YEN2",
+   "KEY_PRTSCR",     "KEY_PAUSE",      "KEY_ABNT_C1",    "KEY_YEN",
+   "KEY_KANA",       "KEY_CONVERT",    "KEY_NOCONVERT",  "KEY_AT",
+   "KEY_CIRCUMFLEX", "KEY_COLON2",     "KEY_KANJI",
+   "KEY_EQUALS_PAD", "KEY_BACKQUOTE",  "KEY_SEMICOLON",  "KEY_COMMAND",
    "KEY_LSHIFT",     "KEY_RSHIFT",     "KEY_LCONTROL",   "KEY_RCONTROL",
    "KEY_ALT",        "KEY_ALTGR",      "KEY_LWIN",       "KEY_RWIN",
    "KEY_MENU",       "KEY_SCRLOCK",    "KEY_NUMLOCK",    "KEY_CAPSLOCK",
@@ -70,63 +77,43 @@ unsigned short orig_key_ascii_table[KEY_MAX];
 unsigned short orig_key_capslock_table[KEY_MAX];
 unsigned short orig_key_shift_table[KEY_MAX];
 unsigned short orig_key_control_table[KEY_MAX];
-unsigned short orig_key_altgr_table[KEY_MAX];
+unsigned short orig_key_altgr_lower_table[KEY_MAX];
+unsigned short orig_key_altgr_upper_table[KEY_MAX];
 unsigned short orig_key_accent1_lower_table[KEY_MAX];
 unsigned short orig_key_accent1_upper_table[KEY_MAX];
-unsigned short orig_key_accent1_shift_lower_table[KEY_MAX];
-unsigned short orig_key_accent1_shift_upper_table[KEY_MAX];
 unsigned short orig_key_accent2_lower_table[KEY_MAX];
 unsigned short orig_key_accent2_upper_table[KEY_MAX];
-unsigned short orig_key_accent2_shift_lower_table[KEY_MAX];
-unsigned short orig_key_accent2_shift_upper_table[KEY_MAX];
+unsigned short orig_key_accent3_lower_table[KEY_MAX];
+unsigned short orig_key_accent3_upper_table[KEY_MAX];
+unsigned short orig_key_accent4_lower_table[KEY_MAX];
+unsigned short orig_key_accent4_upper_table[KEY_MAX];
 
 
 unsigned short my_key_ascii_table[KEY_MAX];
 unsigned short my_key_capslock_table[KEY_MAX];
 unsigned short my_key_shift_table[KEY_MAX];
 unsigned short my_key_control_table[KEY_MAX];
-unsigned short my_key_altgr_table[KEY_MAX];
+unsigned short my_key_altgr_lower_table[KEY_MAX];
+unsigned short my_key_altgr_upper_table[KEY_MAX];
 unsigned short my_key_accent1_lower_table[KEY_MAX];
 unsigned short my_key_accent1_upper_table[KEY_MAX];
-unsigned short my_key_accent1_shift_lower_table[KEY_MAX];
-unsigned short my_key_accent1_shift_upper_table[KEY_MAX];
 unsigned short my_key_accent2_lower_table[KEY_MAX];
 unsigned short my_key_accent2_upper_table[KEY_MAX];
-unsigned short my_key_accent2_shift_lower_table[KEY_MAX];
-unsigned short my_key_accent2_shift_upper_table[KEY_MAX];
+unsigned short my_key_accent3_lower_table[KEY_MAX];
+unsigned short my_key_accent3_upper_table[KEY_MAX];
+unsigned short my_key_accent4_lower_table[KEY_MAX];
+unsigned short my_key_accent4_upper_table[KEY_MAX];
 
 
-#if (!defined ALLEGRO_DOS) && (!defined ALLEGRO_WINDOWS) && (!defined ALLEGRO_WITH_XWINDOWS)
+char keyboard_name[64] = EMPTY_STRING;
 
-   /* if this platform isn't using pckeys.c, we declare these ourselves */
-   int _key_accent1 = 0;
-   int _key_accent2 = 0;
-
-   int _key_standard_kb;
-
-   unsigned short *_key_ascii_table;
-   unsigned short *_key_capslock_table;
-   unsigned short *_key_shift_table;
-   unsigned short *_key_control_table;
-   unsigned short *_key_altgr_table;
-   unsigned short *_key_accent1_lower_table;
-   unsigned short *_key_accent1_upper_table;
-   unsigned short *_key_accent1_shift_lower_table;
-   unsigned short *_key_accent1_shift_upper_table;
-   unsigned short *_key_accent2_lower_table;
-   unsigned short *_key_accent2_upper_table;
-   unsigned short *_key_accent2_shift_lower_table;
-   unsigned short *_key_accent2_shift_upper_table;
-
-#endif
-
-
-char keyboard_name[64] = "";
-
-char config_file[256] = "";
+#define FILENAME_LENGTH  512
+char config_file[FILENAME_LENGTH] = EMPTY_STRING;
 
 unsigned short *editor_table;
 
+int split_altgr = FALSE;
+
 int codepage;
 
 int loader(void);
@@ -135,6 +122,8 @@ int quitter(void);
 int editor(void);
 int accenter1(void);
 int accenter2(void);
+int accenter3(void);
+int accenter4(void);
 int tester(void);
 
 
@@ -155,14 +144,15 @@ MENU edit_menu[] =
    { "Shifted",               editor,     NULL,       0,    NULL  },
    { "Control",               editor,     NULL,       0,    NULL  },
    { "Alt+Gr",                editor,     NULL,       0,    NULL  },
+   { "Alt+Gr (caps)",         editor,     NULL,       0,    NULL  },
    { "Accent1",               editor,     NULL,       0,    NULL  },
    { "Accent1 (caps)",        editor,     NULL,       0,    NULL  },
-   { "Accent1+shift",         editor,     NULL,       0,    NULL  },
-   { "Accent1+shift (caps)",  editor,     NULL,       0,    NULL  },
    { "Accent2",               editor,     NULL,       0,    NULL  },
    { "Accent2 (caps)",        editor,     NULL,       0,    NULL  },
-   { "Accent2+shift",         editor,     NULL,       0,    NULL  },
-   { "Accent2+shift (caps)",  editor,     NULL,       0,    NULL  },
+   { "Accent3",               editor,     NULL,       0,    NULL  },
+   { "Accent3 (caps)",        editor,     NULL,       0,    NULL  },
+   { "Accent4",               editor,     NULL,       0,    NULL  },
+   { "Accent4 (caps)",        editor,     NULL,       0,    NULL  },
    { NULL,                    NULL,       NULL,       0,    NULL  }
 };
 
@@ -171,6 +161,8 @@ MENU misc_menu[] =
 {
    { "Accent #&1",            accenter1,  NULL,       0,    NULL  },
    { "Accent #&2",            accenter2,  NULL,       0,    NULL  },
+   { "Accent #&3",            accenter3,  NULL,       0,    NULL  },
+   { "Accent #&4",            accenter4,  NULL,       0,    NULL  },
    { "",                      NULL,       NULL,       0,    NULL  },
    { "&Test\t(ctrl+T)",       tester,     NULL,       0,    NULL  },
    { NULL,                    NULL,       NULL,       0,    NULL  }
@@ -203,6 +195,7 @@ DIALOG main_dlg[] =
    { d_ctext_proc,      320,  140,  0,    0,    0,    8,    0,       0,          0,             0,       "Allegro Keyboard Setup Utility", NULL, NULL  },
    { d_ctext_proc,      320,  180,  0,    0,    0,    8,    0,       0,          0,             0,       "Version " ALLEGRO_VERSION_STR ", " ALLEGRO_PLATFORM_STR, NULL, NULL  },
    { d_ctext_proc,      320,  250,  0,    0,    0,    8,    0,       0,          0,             0,       "By Shawn Hargreaves, " ALLEGRO_DATE_STR, NULL, NULL  },
+   { d_ctext_proc,      320,  270,  0,    0,    0,    8,    0,       0,          0,             0,       "Modified by Peter Pavlovi\xC4\x8D", NULL, NULL },
    { d_text_proc,       136,  380,  0,    0,    0,    8,    0,       0,          0,             0,       "Keyboard mapping name:", NULL, NULL  },
    { d_edit_proc,       320,  380,  192,  8,    0,    16,   0,       0,          23,            0,       keyboard_name, NULL, NULL  },
    { NULL,              0,    0,    0,    0,    0,    0,    0,       0,          0,             0,       NULL,    NULL, NULL }
@@ -227,11 +220,11 @@ void load_table(unsigned short *table, char *section)
 /* handle the load command */
 int loader()
 {
-   char buf[256];
+   char buf[FILENAME_LENGTH];
 
    strcpy(buf, config_file);
 
-   if (file_select("Load Keyboard Config", buf, "CFG")) {
+   if (file_select_ex("Load Keyboard Config", buf, "CFG", sizeof(buf), 0, 0)) {
       strlwr(buf);
       strcpy(config_file, buf);
 
@@ -239,15 +232,16 @@ int loader()
       memcpy(_key_capslock_table, orig_key_capslock_table, sizeof(orig_key_capslock_table));
       memcpy(_key_shift_table, orig_key_shift_table, sizeof(orig_key_shift_table));
       memcpy(_key_control_table, orig_key_control_table, sizeof(orig_key_control_table));
-      memcpy(_key_altgr_table, orig_key_altgr_table, sizeof(orig_key_altgr_table));
+      memcpy(_key_altgr_lower_table, orig_key_altgr_lower_table, sizeof(orig_key_altgr_lower_table));
+      memcpy(_key_altgr_upper_table, orig_key_altgr_upper_table, sizeof(orig_key_altgr_upper_table));
       memcpy(_key_accent1_lower_table, orig_key_accent1_lower_table, sizeof(orig_key_accent1_lower_table));
       memcpy(_key_accent1_upper_table, orig_key_accent1_upper_table, sizeof(orig_key_accent1_upper_table));
-      memcpy(_key_accent1_shift_lower_table, orig_key_accent1_shift_lower_table, sizeof(orig_key_accent1_shift_lower_table));
-      memcpy(_key_accent1_shift_upper_table, orig_key_accent1_shift_upper_table, sizeof(orig_key_accent1_shift_upper_table));
       memcpy(_key_accent2_lower_table, orig_key_accent2_lower_table, sizeof(orig_key_accent2_lower_table));
       memcpy(_key_accent2_upper_table, orig_key_accent2_upper_table, sizeof(orig_key_accent2_upper_table));
-      memcpy(_key_accent2_shift_lower_table, orig_key_accent2_shift_lower_table, sizeof(orig_key_accent2_shift_lower_table));
-      memcpy(_key_accent2_shift_upper_table, orig_key_accent2_shift_upper_table, sizeof(orig_key_accent2_shift_upper_table));
+      memcpy(_key_accent3_lower_table, orig_key_accent3_lower_table, sizeof(orig_key_accent3_lower_table));
+      memcpy(_key_accent3_upper_table, orig_key_accent3_upper_table, sizeof(orig_key_accent3_upper_table));
+      memcpy(_key_accent4_lower_table, orig_key_accent4_lower_table, sizeof(orig_key_accent4_lower_table));
+      memcpy(_key_accent4_upper_table, orig_key_accent4_upper_table, sizeof(orig_key_accent4_upper_table));
 
       push_config_state();
       set_config_file(buf);
@@ -256,18 +250,30 @@ int loader()
       load_table(_key_capslock_table,            "key_capslock");
       load_table(_key_shift_table,               "key_shift");
       load_table(_key_control_table,             "key_control");
-      load_table(_key_altgr_table,               "key_altgr");
+
+      load_table(_key_altgr_lower_table,         "key_altgr");
+      load_table(_key_altgr_upper_table,         "key_altgr");
+
+      load_table(_key_altgr_lower_table,         "key_altgr_lower");
+      load_table(_key_altgr_upper_table,         "key_altgr_upper");
+
       load_table(_key_accent1_lower_table,       "key_accent1_lower");
       load_table(_key_accent1_upper_table,       "key_accent1_upper");
-      load_table(_key_accent1_shift_lower_table, "key_accent1_shift_lower");
-      load_table(_key_accent1_shift_upper_table, "key_accent1_shift_upper");
       load_table(_key_accent2_lower_table,       "key_accent2_lower");
       load_table(_key_accent2_upper_table,       "key_accent2_upper");
-      load_table(_key_accent2_shift_lower_table, "key_accent2_shift_lower");
-      load_table(_key_accent2_shift_upper_table, "key_accent2_shift_upper");
+      load_table(_key_accent3_lower_table,       "key_accent3_lower");
+      load_table(_key_accent3_upper_table,       "key_accent3_upper");
+      load_table(_key_accent4_lower_table,       "key_accent4_lower");
+      load_table(_key_accent4_upper_table,       "key_accent4_upper");
 
       _key_accent1 = get_config_int("key_escape", "accent1", 0);
       _key_accent2 = get_config_int("key_escape", "accent2", 0);
+      _key_accent3 = get_config_int("key_escape", "accent3", 0);
+      _key_accent4 = get_config_int("key_escape", "accent4", 0);
+      _key_accent1_flag = get_config_int("key_escape", "accent1_flag", 0);
+      _key_accent2_flag = get_config_int("key_escape", "accent2_flag", 0);
+      _key_accent3_flag = get_config_int("key_escape", "accent3_flag", 0);
+      _key_accent4_flag = get_config_int("key_escape", "accent4_flag", 0);
 
       strcpy(keyboard_name, get_config_string(NULL, "keyboard_name", ""));
       main_dlg[11].d2 = strlen(keyboard_name);
@@ -300,11 +306,11 @@ void save_table(unsigned short *table, unsigned short *origtable, char *section)
 /* handle the save command */
 int saver()
 {
-   char buf[256];
+   char buf[FILENAME_LENGTH];
 
    strcpy(buf, config_file);
 
-   if (file_select("Save Keyboard Config", buf, "CFG")) {
+   if (file_select_ex("Save Keyboard Config", buf, "CFG", sizeof(buf), 0, 0)) {
       if ((stricmp(config_file, buf) != 0) && (exists(buf))) {
 	 if (alert("Overwrite existing file?", NULL, NULL, "Yes", "Cancel", 'y', 27) != 1)
 	    return D_REDRAW;
@@ -320,20 +326,34 @@ int saver()
 
       set_config_int("key_escape", "accent1", _key_accent1);
       set_config_int("key_escape", "accent2", _key_accent2);
+      set_config_int("key_escape", "accent3", _key_accent3);
+      set_config_int("key_escape", "accent4", _key_accent4);
+      set_config_int("key_escape", "accent1_flag", _key_accent1_flag);
+      set_config_int("key_escape", "accent2_flag", _key_accent2_flag);
+      set_config_int("key_escape", "accent3_flag", _key_accent3_flag);
+      set_config_int("key_escape", "accent4_flag", _key_accent4_flag);
 
       save_table(_key_ascii_table,               orig_key_ascii_table,               "key_ascii");
       save_table(_key_capslock_table,            orig_key_capslock_table,            "key_capslock");
       save_table(_key_shift_table,               orig_key_shift_table,               "key_shift");
       save_table(_key_control_table,             orig_key_control_table,             "key_control");
-      save_table(_key_altgr_table,               orig_key_altgr_table,               "key_altgr");
+
+      if (split_altgr) {
+         save_table(_key_altgr_lower_table,      orig_key_altgr_lower_table,         "key_altgr_lower");
+         save_table(_key_altgr_upper_table,      orig_key_altgr_upper_table,         "key_altgr_upper");
+      }
+      else {
+         save_table(_key_altgr_lower_table,      orig_key_altgr_lower_table,         "key_altgr");
+      }
+
       save_table(_key_accent1_lower_table,       orig_key_accent1_lower_table,       "key_accent1_lower");
       save_table(_key_accent1_upper_table,       orig_key_accent1_upper_table,       "key_accent1_upper");
-      save_table(_key_accent1_shift_lower_table, orig_key_accent1_shift_lower_table, "key_accent1_shift_lower");
-      save_table(_key_accent1_shift_upper_table, orig_key_accent1_shift_upper_table, "key_accent1_shift_upper");
       save_table(_key_accent2_lower_table,       orig_key_accent2_lower_table,       "key_accent2_lower");
       save_table(_key_accent2_upper_table,       orig_key_accent2_upper_table,       "key_accent2_upper");
-      save_table(_key_accent2_shift_lower_table, orig_key_accent2_shift_lower_table, "key_accent2_shift_lower");
-      save_table(_key_accent2_shift_upper_table, orig_key_accent2_shift_upper_table, "key_accent2_shift_upper");
+      save_table(_key_accent3_lower_table,       orig_key_accent3_lower_table,       "key_accent3_lower");
+      save_table(_key_accent3_upper_table,       orig_key_accent3_upper_table,       "key_accent3_upper");
+      save_table(_key_accent4_lower_table,       orig_key_accent4_lower_table,       "key_accent4_lower");
+      save_table(_key_accent4_upper_table,       orig_key_accent4_upper_table,       "key_accent4_upper");
 
       pop_config_state();
    }
@@ -378,16 +398,15 @@ int ascii_proc(int msg, DIALOG *d, int c)
 
       rectfill(screen, d->x+1, d->y+1, d->x+d->w-1, d->y+d->h-1, bg);
       rect(screen, d->x, d->y, d->x+d->w, d->y+d->h, fg);
-      text_mode(-1);
 
-      textprintf(screen, font, d->x+4, d->y+4, mg, "%02X", d->d1);
+      textprintf_ex(screen, font, d->x+4, d->y+4, mg, -1, "%02X", d->d1);
 
       k = (codepage << 8) | d->d1;
 
       if (k >= ' ')
-	 textprintf_centre(screen, font, d->x+d->w-8, d->y+4, fg, "%c", k);
+	 textprintf_centre_ex(screen, font, d->x+d->w-8, d->y+4, fg, -1, "%c", k);
       else
-	 textprintf_centre(screen, font, d->x+d->w-8, d->y+4, fg, "%s", ascii_name[k]);
+	 textprintf_centre_ex(screen, font, d->x+d->w-8, d->y+4, fg, -1, "%s", ascii_name[k]);
 
       return D_O_K;
    }
@@ -418,9 +437,8 @@ int codepage_proc(int msg, DIALOG *d, int c)
 
       rectfill(screen, d->x+1, d->y+1, d->x+d->w-1, d->y+d->h-1, bg);
       rect(screen, d->x, d->y, d->x+d->w, d->y+d->h, 255);
-      text_mode(-1);
 
-      textprintf_centre(screen, font, d->x+d->w/2, d->y+4, fg, "%04X", d->d1<<8);
+      textprintf_centre_ex(screen, font, d->x+d->w/2, d->y+4, fg, -1, "%04X", d->d1<<8);
 
       return D_O_K;
    }
@@ -556,8 +574,8 @@ DIALOG editor_dlg[] =
    /* (dialog proc)     (x)   (y)   (w)   (h)   (fg)  (bg)  (key)    (flags)     (d1)           (d2)     (dp)                 (dp2) (dp3) */
    { d_clear_proc,      0,    0,    0,    0,    0,    8,    0,       0,          0,             0,       NULL,                NULL, NULL  },
    { d_ctext_proc,      320,  6,    0,    0,    0,    8,    0,       0,          0,             0,       NULL,                NULL, NULL  },
-   { keymap_list_proc,  16,   32,   608,  403,  255,  16,   0,       D_EXIT,     0,             0,       keymap_list_getter,  NULL, NULL  },
-   { d_button_proc,     280,  448,  80,   16,   255,  16,   0,       D_EXIT,     0,             0,       "Exit",              NULL, NULL  },
+   { keymap_list_proc,  16,   32,   609,  404,  255,  16,   0,       D_EXIT,     0,             0,       keymap_list_getter,  NULL, NULL  },
+   { d_button_proc,     280,  448,  81,   17,   255,  16,   0,       D_EXIT,     0,             0,       "Exit",              NULL, NULL  },
    { NULL,              0,    0,    0,    0,    0,    0,    0,       0,          0,             0,       NULL,                NULL, NULL  }
 };
 
@@ -595,9 +613,15 @@ DIALOG accent_dlg[] =
    /* (dialog proc)     (x)   (y)   (w)   (h)   (fg)  (bg)  (key)    (flags)     (d1)           (d2)     (dp)                 (dp2) (dp3) */
    { d_clear_proc,      0,    0,    0,    0,    0,    8,    0,       0,          0,             0,       NULL,                NULL, NULL  },
    { d_ctext_proc,      320,  6,    0,    0,    255,  8,    0,       0,          0,             0,       NULL,                NULL, NULL  },
-   { d_list_proc,       16,   32,   608,  403,  255,  16,   0,       D_EXIT,     0,             0,       accent_list_getter,  NULL, NULL  },
-   { d_button_proc,     220,  448,  80,   16,   255,  16,   0,       D_EXIT,     0,             0,       "OK",                NULL, NULL  },
-   { d_button_proc,     340,  448,  80,   16,   255,  16,   0,       D_EXIT,     0,             0,       "Cancel",            NULL, NULL  },
+   { d_list_proc,       16,   32,   609,  341,  255,  16,   0,       D_EXIT,     0,             0,       accent_list_getter,  NULL, NULL  },
+   { d_button_proc,     220,  448,  81,   17,   255,  16,   0,       D_EXIT,     0,             0,       "OK",                NULL, NULL  },
+   { d_button_proc,     340,  448,  81,   17,   255,  16,   0,       D_EXIT,     0,             0,       "Cancel",            NULL, NULL  },
+   { d_box_proc,        16,   380,  609,  56,   255,  16,   0,       0,          0,             0,       NULL,                NULL, NULL  },
+   { d_radio_proc,      36,   413,  51,   11,   255,  16,   0,       0,          0,             0,       "None",              NULL, NULL  },
+   { d_radio_proc,      179,  413,  59,   11,   255,  16,   0,       0,          0,             0,       "Shift",             NULL, NULL  },
+   { d_radio_proc,      346,  413,  75,   11,   255,  16,   0,       0,          0,             0,       "Control",           NULL, NULL  },
+   { d_radio_proc,      516,  413,  43,   11,   255,  16,   0,       0,          0,             0,       "Alt",               NULL, NULL  },
+   { d_ctext_proc,      319,  392,  224,  8,    255,  16,   0,       0,          0,             0,       "Set the accent activator key", NULL, NULL },
    { NULL,              0,    0,    0,    0,    0,    0,    0,       0,          0,             0,       NULL,                NULL, NULL  }
 };
 
@@ -609,21 +633,70 @@ int accenter(char *title, int flag)
    int ret;
 
    accent_dlg[1].dp = title;
-
-   if (flag == KB_ACCENT1_FLAG)
-      accent_dlg[2].d1 = _key_accent1;
-   else
-      accent_dlg[2].d1 = _key_accent2;
-
    accent_dlg[2].d2 = 0;
+   accent_dlg[6].flags = accent_dlg[7].flags = 0;
+   accent_dlg[8].flags = accent_dlg[9].flags = 0;
+
+   if (flag == KB_ACCENT1_FLAG) {
+      accent_dlg[2].d1 = _key_accent1;
+      if (_key_accent1_flag == 0) accent_dlg[6].flags = D_SELECTED;
+      else if (_key_accent1_flag == 1) accent_dlg[7].flags = D_SELECTED;
+      else if (_key_accent1_flag == 2) accent_dlg[8].flags = D_SELECTED;
+      else if (_key_accent1_flag == 4) accent_dlg[9].flags = D_SELECTED;
+   }
+   else if (flag == KB_ACCENT2_FLAG) {
+      accent_dlg[2].d1 = _key_accent2;
+      if (_key_accent2_flag == 0) accent_dlg[6].flags = D_SELECTED;
+      else if (_key_accent2_flag == 1) accent_dlg[7].flags = D_SELECTED;
+      else if (_key_accent2_flag == 2) accent_dlg[8].flags = D_SELECTED;
+      else if (_key_accent2_flag == 4) accent_dlg[9].flags = D_SELECTED;
+   }
+   else if (flag == KB_ACCENT3_FLAG) {
+      accent_dlg[2].d1 = _key_accent3;
+      if (_key_accent3_flag == 0) accent_dlg[6].flags = D_SELECTED;
+      else if (_key_accent3_flag == 1) accent_dlg[7].flags = D_SELECTED;
+      else if (_key_accent3_flag == 2) accent_dlg[8].flags = D_SELECTED;
+      else if (_key_accent3_flag == 4) accent_dlg[9].flags = D_SELECTED;
+   }
+   else if (flag == KB_ACCENT4_FLAG) {
+      accent_dlg[2].d1 = _key_accent4;
+      if (_key_accent4_flag == 0) accent_dlg[6].flags = D_SELECTED;
+      else if (_key_accent4_flag == 1) accent_dlg[7].flags = D_SELECTED;
+      else if (_key_accent4_flag == 2) accent_dlg[8].flags = D_SELECTED;
+      else if (_key_accent4_flag == 4) accent_dlg[9].flags = D_SELECTED;
+   }
 
    ret = do_dialog(accent_dlg, -1);
 
    if ((ret == 2) || (ret == 3)) {
-      if (flag == KB_ACCENT1_FLAG)
+      if (flag == KB_ACCENT1_FLAG) {
 	 _key_accent1 = accent_dlg[2].d1;
-      else
+	 if (accent_dlg[6].flags & D_SELECTED) _key_accent1_flag = 0;
+	 else if (accent_dlg[7].flags & D_SELECTED) _key_accent1_flag = 1;
+	 else if (accent_dlg[8].flags & D_SELECTED) _key_accent1_flag = 2;
+	 else if (accent_dlg[9].flags & D_SELECTED) _key_accent1_flag = 4;
+      }
+      else if (flag == KB_ACCENT2_FLAG) {
 	 _key_accent2 = accent_dlg[2].d1;
+	 if (accent_dlg[6].flags & D_SELECTED) _key_accent2_flag = 0;
+	 else if (accent_dlg[7].flags & D_SELECTED) _key_accent2_flag = 1;
+	 else if (accent_dlg[8].flags & D_SELECTED) _key_accent2_flag = 2;
+	 else if (accent_dlg[9].flags & D_SELECTED) _key_accent2_flag = 4;
+      }
+      else if (flag == KB_ACCENT3_FLAG) {
+	 _key_accent3 = accent_dlg[2].d1;
+	 if (accent_dlg[6].flags & D_SELECTED) _key_accent3_flag = 0;
+	 else if (accent_dlg[7].flags & D_SELECTED) _key_accent3_flag = 1;
+	 else if (accent_dlg[8].flags & D_SELECTED) _key_accent3_flag = 2;
+	 else if (accent_dlg[9].flags & D_SELECTED) _key_accent3_flag = 4;
+      }
+      else if (flag == KB_ACCENT4_FLAG) {
+	 _key_accent4 = accent_dlg[2].d1;
+	 if (accent_dlg[6].flags & D_SELECTED) _key_accent4_flag = 0;
+	 else if (accent_dlg[7].flags & D_SELECTED) _key_accent4_flag = 1;
+	 else if (accent_dlg[8].flags & D_SELECTED) _key_accent4_flag = 2;
+	 else if (accent_dlg[9].flags & D_SELECTED) _key_accent4_flag = 4;
+      }
    }
 
    return D_REDRAW;
@@ -645,6 +718,20 @@ int accenter2()
 
 
 
+int accenter3()
+{
+   return accenter("Set Accent Key #3", KB_ACCENT3_FLAG);
+}
+
+
+
+int accenter4()
+{
+   return accenter("Set Accent Key #4", KB_ACCENT4_FLAG);
+}
+
+
+
 /* handle the test command */
 int tester()
 {
@@ -654,12 +741,10 @@ int tester()
    show_mouse(NULL);
 
    acquire_screen();
-   clear_to_color(screen, 8);
-
-   text_mode(8);
+   clear_to_color(screen, palette_color[8]);
 
    for (i=0; i<=KEY_MAX; i++)
-      textout(screen, font, scancode_name[i], 32+(i%4)*160, 88+(i/4)*14, 255);
+      textout_ex(screen, font, scancode_name[i], 32+(i%4)*160, 60+(i/4)*14, palette_color[255], palette_color[8]);
 
    release_screen();
 
@@ -675,7 +760,7 @@ int tester()
       acquire_screen();
 
       for (i=0; i<KEY_MAX; i++)
-	 textout(screen, font, key[i] ? "*" : " ", 16+(i%4)*160, 88+(i/4)*14, 255);
+	 textout_ex(screen, font, key[i] ? "*" : " ", 16+(i%4)*160, 60+(i/4)*14, palette_color[255], palette_color[8]);
 
       buf[0] = 0;
 
@@ -697,6 +782,9 @@ int tester()
       if (key_shifts & KB_MENU_FLAG)
 	 strcat(buf, "menu ");
 
+      if (key_shifts & KB_COMMAND_FLAG)
+	 strcat(buf, "command ");
+
       if (key_shifts & KB_SCROLOCK_FLAG)
 	 strcat(buf, "scrolock ");
 
@@ -710,21 +798,21 @@ int tester()
 	 strcat(buf, "inaltseq ");
 
       if (key_shifts & KB_ACCENT1_FLAG)
-	 strcat(buf, "accent1 ");
-
-      if (key_shifts & KB_ACCENT1_S_FLAG)
-	 strcat(buf, "accent1+shift ");
+	 strcpy(buf, "accent1 ");
 
       if (key_shifts & KB_ACCENT2_FLAG)
-	 strcat(buf, "accent2 ");
+	 strcpy(buf, "accent2 ");
 
-      if (key_shifts & KB_ACCENT2_S_FLAG)
-	 strcat(buf, "accent2+shift ");
+      if (key_shifts & KB_ACCENT3_FLAG)
+	 strcpy(buf, "accent3 ");
+
+      if (key_shifts & KB_ACCENT4_FLAG)
+	 strcpy(buf, "accent4 ");
 
       while (strlen(buf) < 128)
 	 strcat(buf, " ");
 
-      textout(screen, font, buf, 0, 0, 255);
+      textout_ex(screen, font, buf, 0, 0, palette_color[255], palette_color[8]);
 
       release_screen();
 
@@ -732,7 +820,7 @@ int tester()
 	 a = ureadkey(&i);
 	 if (!a)
 	    a = ' ';
-	 textprintf(screen, font, 32, 48, 255, "ureadkey() returns scancode 0x%02X, U+0x%04X, '%c'", i, a, a);
+	 textprintf_ex(screen, font, 32, 34, palette_color[255], palette_color[8], "ureadkey() returns scancode 0x%02X, U+0x%04X, '%c'", i, a, a);
       }
 
    } while ((!key[KEY_ESC]) && (!mouse_b));
@@ -750,12 +838,26 @@ int tester()
 
 
 
-int main()
+int main(int argc, char *argv[])
 {
-   static char config_override[] = "keyboard = \n";
+   static char config_override[] = "[system]\nkeyboard = \n";
+   RGB black_rgb = {0, 0, 0, 0};
    DATAFILE *font_data;
+   int c;
+
+   if (allegro_init() != 0)
+      exit(EXIT_FAILURE);
+
+   if (argc > 1) {
+      if (strcmp(argv[1], "--split-altgr") == 0) {
+         split_altgr = TRUE;
+      }
+      else {
+         allegro_message("Error: unrecognized option\n");
+         exit(EXIT_FAILURE);
+      }
+   }
 
-   allegro_init();
    install_mouse();
    install_timer();
 
@@ -768,63 +870,95 @@ int main()
    memcpy(orig_key_capslock_table, _key_capslock_table, sizeof(orig_key_capslock_table));
    memcpy(orig_key_shift_table, _key_shift_table, sizeof(orig_key_shift_table));
    memcpy(orig_key_control_table, _key_control_table, sizeof(orig_key_control_table));
-   memcpy(orig_key_altgr_table, _key_altgr_table, sizeof(orig_key_altgr_table));
+   memcpy(orig_key_altgr_lower_table, _key_altgr_lower_table, sizeof(orig_key_altgr_lower_table));
+   memcpy(orig_key_altgr_upper_table, _key_altgr_upper_table, sizeof(orig_key_altgr_upper_table));
    memcpy(orig_key_accent1_lower_table, _key_accent1_lower_table, sizeof(orig_key_accent1_lower_table));
    memcpy(orig_key_accent1_upper_table, _key_accent1_upper_table, sizeof(orig_key_accent1_upper_table));
-   memcpy(orig_key_accent1_shift_lower_table, _key_accent1_shift_lower_table, sizeof(orig_key_accent1_shift_lower_table));
-   memcpy(orig_key_accent1_shift_upper_table, _key_accent1_shift_upper_table, sizeof(orig_key_accent1_shift_upper_table));
    memcpy(orig_key_accent2_lower_table, _key_accent2_lower_table, sizeof(orig_key_accent2_lower_table));
    memcpy(orig_key_accent2_upper_table, _key_accent2_upper_table, sizeof(orig_key_accent2_upper_table));
-   memcpy(orig_key_accent2_shift_lower_table, _key_accent2_shift_lower_table, sizeof(orig_key_accent2_shift_lower_table));
-   memcpy(orig_key_accent2_shift_upper_table, _key_accent2_shift_upper_table, sizeof(orig_key_accent2_shift_upper_table));
+   memcpy(orig_key_accent3_lower_table, _key_accent3_lower_table, sizeof(orig_key_accent3_lower_table));
+   memcpy(orig_key_accent3_upper_table, _key_accent3_upper_table, sizeof(orig_key_accent3_upper_table));
+   memcpy(orig_key_accent4_lower_table, _key_accent4_lower_table, sizeof(orig_key_accent4_lower_table));
+   memcpy(orig_key_accent4_upper_table, _key_accent4_upper_table, sizeof(orig_key_accent4_upper_table));
 
    memcpy(my_key_ascii_table, _key_ascii_table, sizeof(my_key_ascii_table));
    memcpy(my_key_capslock_table, _key_capslock_table, sizeof(my_key_capslock_table));
    memcpy(my_key_shift_table, _key_shift_table, sizeof(my_key_shift_table));
    memcpy(my_key_control_table, _key_control_table, sizeof(my_key_control_table));
-   memcpy(my_key_altgr_table, _key_altgr_table, sizeof(my_key_altgr_table));
+   memcpy(my_key_altgr_lower_table, _key_altgr_lower_table, sizeof(my_key_altgr_lower_table));
+   memcpy(my_key_altgr_upper_table, _key_altgr_upper_table, sizeof(my_key_altgr_upper_table));
    memcpy(my_key_accent1_lower_table, _key_accent1_lower_table, sizeof(my_key_accent1_lower_table));
    memcpy(my_key_accent1_upper_table, _key_accent1_upper_table, sizeof(my_key_accent1_upper_table));
-   memcpy(my_key_accent1_shift_lower_table, _key_accent1_shift_lower_table, sizeof(my_key_accent1_shift_lower_table));
-   memcpy(my_key_accent1_shift_upper_table, _key_accent1_shift_upper_table, sizeof(my_key_accent1_shift_upper_table));
    memcpy(my_key_accent2_lower_table, _key_accent2_lower_table, sizeof(my_key_accent2_lower_table));
    memcpy(my_key_accent2_upper_table, _key_accent2_upper_table, sizeof(my_key_accent2_upper_table));
-   memcpy(my_key_accent2_shift_lower_table, _key_accent2_shift_lower_table, sizeof(my_key_accent2_shift_lower_table));
-   memcpy(my_key_accent2_shift_upper_table, _key_accent2_shift_upper_table, sizeof(my_key_accent2_shift_upper_table));
+   memcpy(my_key_accent3_lower_table, _key_accent3_lower_table, sizeof(my_key_accent3_lower_table));
+   memcpy(my_key_accent3_upper_table, _key_accent3_upper_table, sizeof(my_key_accent3_upper_table));
+   memcpy(my_key_accent4_lower_table, _key_accent4_lower_table, sizeof(my_key_accent4_lower_table));
+   memcpy(my_key_accent4_upper_table, _key_accent4_upper_table, sizeof(my_key_accent4_upper_table));
 
    _key_ascii_table = my_key_ascii_table;
    _key_capslock_table = my_key_capslock_table;
    _key_shift_table = my_key_shift_table;
    _key_control_table = my_key_control_table;
-   _key_altgr_table = my_key_altgr_table;
+   _key_altgr_lower_table = my_key_altgr_lower_table;
+   _key_altgr_upper_table = my_key_altgr_upper_table;
    _key_accent1_lower_table = my_key_accent1_lower_table;
    _key_accent1_upper_table = my_key_accent1_upper_table;
-   _key_accent1_shift_lower_table = my_key_accent1_shift_lower_table;
-   _key_accent1_shift_upper_table = my_key_accent1_shift_upper_table;
    _key_accent2_lower_table = my_key_accent2_lower_table;
    _key_accent2_upper_table = my_key_accent2_upper_table;
-   _key_accent2_shift_lower_table = my_key_accent2_shift_lower_table;
-   _key_accent2_shift_upper_table = my_key_accent2_shift_upper_table;
+   _key_accent3_lower_table = my_key_accent3_lower_table;
+   _key_accent3_upper_table = my_key_accent3_upper_table;
+   _key_accent4_lower_table = my_key_accent4_lower_table;
+   _key_accent4_upper_table = my_key_accent4_upper_table;
 
    edit_menu[0].dp = _key_ascii_table;
    edit_menu[1].dp = _key_capslock_table;
    edit_menu[2].dp = _key_shift_table;
    edit_menu[3].dp = _key_control_table;
-   edit_menu[4].dp = _key_altgr_table;
-   edit_menu[5].dp = _key_accent1_lower_table;
-   edit_menu[6].dp = _key_accent1_upper_table;
-   edit_menu[7].dp = _key_accent1_shift_lower_table;
-   edit_menu[8].dp = _key_accent1_shift_upper_table;
-   edit_menu[9].dp = _key_accent2_lower_table;
-   edit_menu[10].dp = _key_accent2_upper_table;
-   edit_menu[11].dp = _key_accent2_shift_lower_table;
-   edit_menu[12].dp = _key_accent2_shift_upper_table;
+   edit_menu[4].dp = _key_altgr_lower_table;
+   edit_menu[5].dp = _key_altgr_upper_table;
+   edit_menu[6].dp = _key_accent1_lower_table;
+   edit_menu[7].dp = _key_accent1_upper_table;
+   edit_menu[8].dp = _key_accent2_lower_table;
+   edit_menu[9].dp = _key_accent2_upper_table;
+   edit_menu[10].dp = _key_accent3_lower_table;
+   edit_menu[11].dp = _key_accent3_upper_table;
+   edit_menu[12].dp = _key_accent4_lower_table;
+   edit_menu[13].dp = _key_accent4_upper_table;
+
+   if (!split_altgr)
+      edit_menu[5].flags = D_DISABLED;
+
+   if (set_gfx_mode(GFX_AUTODETECT, 640, 480, 0, 0) != 0) {
+      if (set_gfx_mode(GFX_SAFE, 640, 480, 0, 0) != 0) {
+	 set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
+	 allegro_message("Unable to set any graphic mode\n%s\n", allegro_error);
+	 return 1;
+      }
+   }
 
-   set_gfx_mode(GFX_SAFE, 640, 480, 0, 0);
    set_palette(desktop_palette);
    set_color(0, &black_rgb);
-   gui_fg_color = 255;
-   gui_bg_color = 16;
+   gui_fg_color = palette_color[255];
+   gui_bg_color = palette_color[16];
+
+   /* We set up colors to match screen color depth (in case it changed) */
+   for (c = 0; main_dlg[c].proc; c++) {
+      main_dlg[c].fg = palette_color[main_dlg[c].fg];
+      main_dlg[c].bg = palette_color[main_dlg[c].bg];
+   }
+   for (c = 0; ascii_dlg[c].proc; c++) {
+      ascii_dlg[c].fg = palette_color[ascii_dlg[c].fg];
+      ascii_dlg[c].bg = palette_color[ascii_dlg[c].bg];
+   }
+   for (c = 0; editor_dlg[c].proc; c++) {
+      editor_dlg[c].fg = palette_color[editor_dlg[c].fg];
+      editor_dlg[c].bg = palette_color[editor_dlg[c].bg];
+   }
+   for (c = 0; accent_dlg[c].proc; c++) {
+      accent_dlg[c].fg = palette_color[accent_dlg[c].fg];
+      accent_dlg[c].bg = palette_color[accent_dlg[c].bg];
+   }
 
    _key_standard_kb = FALSE;
 
@@ -841,4 +975,18 @@ int main()
    return 0;
 }
 
+
+#else
+
+
+int main(void)
+{
+   allegro_init();
+   allegro_message("The KEYCONF utility is not needed on this platform.\n");
+   return 1;
+}
+
+
+#endif
+
 END_OF_MAIN();
