@@ -17,9 +17,11 @@
 
 
 #include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
 
 #include "allegro.h"
-#include "allegro/aintern.h"
+#include "allegro/internal/aintern.h"
 
 
 
@@ -28,7 +30,7 @@ char allegro_id[] = "Allegro " ALLEGRO_VERSION_STR ", " ALLEGRO_PLATFORM_STR;
 
 
 /* error message for sound and gfx init routines */
-char allegro_error[256] = EMPTY_STRING;
+char allegro_error[ALLEGRO_ERROR_SIZE] = EMPTY_STRING;
 
 
 /* current system driver */
@@ -50,6 +52,8 @@ GFX_DRIVER *gfx_driver = NULL;
 /* abilities of the current graphics driver */
 int gfx_capabilities = 0;
 
+/* pointer to list of valid video modes */
+GFX_MODE_LIST *gfx_mode_list = NULL;
 
 /* a bitmap structure for accessing the physical screen */
 BITMAP *screen = NULL;
@@ -73,8 +77,6 @@ PALETTE _current_palette;
 
 int _current_palette_changed = 0xFFFFFFFF;
 
-RGB black_rgb = { 0, 0, 0, 0 };
-
 
 PALETTE desktop_palette = 
 {
@@ -173,16 +175,16 @@ int *_gfx_bank = NULL;
 
 /* what OS are we running under? */
 int os_type = OSTYPE_UNKNOWN;
+int os_version = -1;
+int os_revision = -1;
+int os_multitasking = FALSE;
 
 
 /* processor information */
-char cpu_vendor[32] = EMPTY_STRING;
+char cpu_vendor[_AL_CPU_VENDOR_SIZE] = EMPTY_STRING;
 int cpu_family = 0;
 int cpu_model = 0;
-int cpu_fpu = FALSE; 
-int cpu_mmx = FALSE; 
-int cpu_3dnow = FALSE;
-int cpu_cpuid = FALSE; 
+int cpu_capabilities = 0;
 
 
 /* debugging stuff */
@@ -192,21 +194,28 @@ static int debug_trace_virgin = TRUE;
 static FILE *assert_file = NULL;
 static FILE *trace_file = NULL;
 
-static int (*assert_handler)(char *msg) = NULL;
-static int (*trace_handler)(char *msg) = NULL;
+static int (*assert_handler)(AL_CONST char *msg) = NULL;
+static int (*trace_handler)(AL_CONST char *msg) = NULL;
+
+
+/* Module linking system stuff: if an object file is linked in, then its
+ * constructor function is executed; this function should fill in the
+ * data structures below (declared in aintern.h). If the module is not
+ * linked in, then the structure pointers will be null, so we don't need
+ * to bother with that bit of code elsewhere.
+ */
+struct _AL_LINKER_MIDI *_al_linker_midi = NULL;
+struct _AL_LINKER_MOUSE *_al_linker_mouse = NULL;
 
 
 /* dynamic registration system for cleanup code */
-#define MAX_EXIT_FUNCS     16
-
-typedef void (*FUNCPTR)(void);
-
-static FUNCPTR exit_funcs[MAX_EXIT_FUNCS] = 
-{ 
-   NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
-   NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL
+struct al_exit_func {
+   void (*funcptr)(void);
+   struct al_exit_func *next;
 };
 
+static struct al_exit_func *exit_func_list = NULL;
+
 
 
 /* _add_exit_func:
@@ -214,14 +223,19 @@ static FUNCPTR exit_funcs[MAX_EXIT_FUNCS] =
  */
 void _add_exit_func(void (*func)(void))
 {
-   int c;
+   struct al_exit_func *n;
 
-   for (c=0; c<MAX_EXIT_FUNCS; c++) {
-      if ((!exit_funcs[c]) || (exit_funcs[c] == func)) {
-	 exit_funcs[c] = func;
+   for (n = exit_func_list; n; n = n->next)
+      if (n->funcptr == func)
 	 return;
-      }
-   }
+
+   n = malloc(sizeof(struct al_exit_func));
+   if (!n)
+      return;
+
+   n->next = exit_func_list;
+   n->funcptr = func;
+   exit_func_list = n;
 }
 
 
@@ -231,37 +245,44 @@ void _add_exit_func(void (*func)(void))
  */
 void _remove_exit_func(void (*func)(void))
 {
-   int c;
+   struct al_exit_func *iter = exit_func_list, *prev = NULL;
 
-   for (c=0; c<MAX_EXIT_FUNCS; c++) {
-      if (exit_funcs[c] == func) {
-	 while (c<MAX_EXIT_FUNCS-1) {
-	    exit_funcs[c] = exit_funcs[c+1];
-	    c++;
-	 }
-	 exit_funcs[MAX_EXIT_FUNCS-1] = NULL;
-	 break;
+   while (iter) {
+      if (iter->funcptr == func) {
+	 if (prev)
+	    prev->next = iter->next;
+	 else
+	    exit_func_list = iter->next;
+	 free(iter);
+	 return;
       }
+      prev = iter;
+      iter = iter->next;
    }
 }
 
 
 
-/* allegro_init:
+/* install_allegro:
  *  Initialises the Allegro library, activating the system driver.
  */
 int install_allegro(int system_id, int *errno_ptr, int (*atexit_ptr)(void (*func)(void)))
 {
-   char tmp[32];
+   RGB black_rgb = {0, 0, 0, 0};
+   char tmp1[64], tmp2[64];
    int i;
 
-   #ifndef CONSTRUCTOR_FUNCTION
+   #ifndef ALLEGRO_USE_CONSTRUCTOR
       /* call constructor functions manually */
       extern void _initialize_datafile_types();
       extern void _midi_constructor();
+      extern void _mouse_constructor();
 
       _initialize_datafile_types();
       _midi_constructor();
+      _mouse_constructor();
+      _register_bitmap_file_type_init();
+      _register_sample_file_type_init();
    #endif
 
    if (errno_ptr)
@@ -295,10 +316,10 @@ int install_allegro(int system_id, int *errno_ptr, int (*atexit_ptr)(void (*func
    /* nasty stuff to set up the config system before the system driver */
    system_driver = _system_driver_list[0].driver;
 
-   _load_config_text();
+   reload_config_texts(NULL);
 
    if (system_id == SYSTEM_AUTODETECT)
-      system_id = get_config_id(NULL, uconvert_ascii("system", tmp), SYSTEM_AUTODETECT);
+      system_id = get_config_id(uconvert_ascii("system", tmp1), uconvert_ascii("system", tmp2), SYSTEM_AUTODETECT);
 
    system_driver = NULL;
 
@@ -320,16 +341,11 @@ int install_allegro(int system_id, int *errno_ptr, int (*atexit_ptr)(void (*func
       }
    }
 
-   if (!system_driver) {
-      char *msg = get_config_text("Fatal error: unable to activate the Allegro system");
+   if (!system_driver)
+      return -1;
 
-      if (ugetc(allegro_error))
-	 allegro_message("%s\n%s\n", msg, allegro_error);
-      else
-	 allegro_message("%s\n", msg);
-
-      exit(1);
-   }
+   /* disable close button */
+   set_close_button_callback(NULL);
 
    /* detect CPU type */
    check_cpu();
@@ -338,8 +354,6 @@ int install_allegro(int system_id, int *errno_ptr, int (*atexit_ptr)(void (*func
    if (_allegro_count == 0) {
       if (atexit_ptr)
 	 atexit_ptr(allegro_exit);
-      else
-	 atexit(allegro_exit);
    }
 
    _allegro_count++;
@@ -354,18 +368,8 @@ int install_allegro(int system_id, int *errno_ptr, int (*atexit_ptr)(void (*func
  */
 void allegro_exit()
 {
-   int done, c;
-
-   do {
-      done = TRUE;
-
-      for (c=MAX_EXIT_FUNCS-1; c>=0; c--) {
-	 if (exit_funcs[c]) {
-	    (*exit_funcs[c])();
-	    done = FALSE;
-	 }
-      }
-   } while (!done);
+   while (exit_func_list)
+      (*(exit_func_list->funcptr))();
 
    if (system_driver) {
       system_driver->exit();
@@ -384,20 +388,20 @@ void allegro_exit()
 /* allegro_message:
  *  Displays a message in whatever form the current platform requires.
  */
-void allegro_message(char *msg, ...)
+void allegro_message(AL_CONST char *msg, ...)
 {
-   char *buf = malloc(4096);
-   char *tmp = malloc(4096);
-
+   char *buf = malloc(ALLEGRO_MESSAGE_SIZE);
+   char *tmp = malloc(ALLEGRO_MESSAGE_SIZE);
    va_list ap;
+   ASSERT(msg);
    va_start(ap, msg);
-   uvsprintf(buf, msg, ap);
+   uvszprintf(buf, ALLEGRO_MESSAGE_SIZE, msg, ap);
    va_end(ap);
 
    if ((system_driver) && (system_driver->message))
       system_driver->message(buf);
    else
-      fputs(uconvert(buf, U_CURRENT, tmp, U_ASCII_CP, 4096), stdout);
+      fputs(uconvert(buf, U_CURRENT, tmp, U_ASCII_CP, ALLEGRO_MESSAGE_SIZE), stdout);
 
    free(buf);
    free(tmp);
@@ -405,6 +409,43 @@ void allegro_message(char *msg, ...)
 
 
 
+/* get_executable_name:
+ *  Finds out the name of the current executable.
+ */
+void get_executable_name(char *output, int size)
+{
+   ASSERT(system_driver);
+   ASSERT(output);
+
+   if (system_driver->get_executable_name) {
+      system_driver->get_executable_name(output, size);
+   }
+   else {
+      output += usetc(output, '.');
+      output += usetc(output, '/');
+      usetc(output, 0);
+   }
+}
+
+
+
+/* set_close_button_callback:
+ *  Installs a callback function to be called when the close button
+ *  of the window or any equivalent device is activated. Returns 0 on
+ *  success and -1 on failure (e.g. feature not supported).
+ */
+int set_close_button_callback(void (*proc)(void))
+{
+   ASSERT(system_driver);
+
+   if (system_driver->set_close_button_callback)
+      return system_driver->set_close_button_callback(proc);
+
+   return -1;
+}
+
+
+
 /* debug_exit:
  *  Closes the debugging output files.
  */
@@ -429,18 +470,19 @@ static void debug_exit(void)
 
 
 /* al_assert:
- *  Raises an assert.
+ *  Raises an assert (uses ASCII strings).
  */
-void al_assert(char *file, int line)
+void al_assert(AL_CONST char *file, int line)
 {
    static int asserted = FALSE;
    int olderr = errno;
-   char buf[80];
+   char buf[128];
    char *s;
 
    if (asserted)
       return;
 
+   /* todo, some day: use snprintf (C99) */
    sprintf(buf, "Assert failed at line %d of %s", line, file);
 
    if (assert_handler) {
@@ -485,17 +527,18 @@ void al_assert(char *file, int line)
 
 
 /* al_trace:
- *  Outputs a trace message.
+ *  Outputs a trace message (uses ASCII strings).
  */
-void al_trace(char *msg, ...)
+void al_trace(AL_CONST char *msg, ...)
 {
    int olderr = errno;
    char buf[512];
    char *s;
 
+   /* todo, some day: use vsnprintf (C99) */
    va_list ap;
    va_start(ap, msg);
-   uvsprintf(buf, msg, ap);
+   vsprintf(buf, msg, ap);
    va_end(ap);
 
    if (trace_handler) {
@@ -518,7 +561,7 @@ void al_trace(char *msg, ...)
    }
 
    if (trace_file) {
-      fwrite(buf, 1, ustrsize(buf), trace_file);
+      fwrite(buf, sizeof(char), strlen(buf), trace_file);
       fflush(trace_file);
    }
 
@@ -530,7 +573,7 @@ void al_trace(char *msg, ...)
 /* register_assert_handler:
  *  Installs a user handler for assert failures.
  */
-void register_assert_handler(int (*handler)(char *msg))
+void register_assert_handler(int (*handler)(AL_CONST char *msg))
 {
    assert_handler = handler;
 }
@@ -540,7 +583,7 @@ void register_assert_handler(int (*handler)(char *msg))
 /* register_trace_handler:
  *  Installs a user handler for trace output.
  */
-void register_trace_handler(int (*handler)(char *msg))
+void register_trace_handler(int (*handler)(AL_CONST char *msg))
 {
    trace_handler = handler;
 }
@@ -596,8 +639,9 @@ SYSTEM_DRIVER system_none =
    "Agnostic",
    sys_none_init,
    sys_none_exit,
-   NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
-   NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
+   NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
+   NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
+   NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
    sys_no_driver, sys_no_driver, sys_no_driver, sys_no_driver,
    sys_no_driver, sys_no_driver, sys_no_driver
 };
