@@ -36,6 +36,8 @@
 
 #define EXE_NAME_UNKNOWN "./UNKNOWN"
 
+#define YIELD_TIME			30000
+
 
 
 status_t ignore_result = 0;
@@ -92,10 +94,10 @@ extern "C" void _be_sys_get_executable_name(char *output, int size)
    cookie = 0;
 
    if (get_next_image_info(0, &cookie, &info) == B_NO_ERROR) {
-      _al_sane_strncpy(output, info.name, size);
+      strncpy(output, info.name, size);
    }
    else {
-      _al_sane_strncpy(output, EXE_NAME_UNKNOWN, size);
+      strncpy(output, EXE_NAME_UNKNOWN, size);
    }
 
    output[size-1] = '\0';
@@ -116,7 +118,7 @@ static int32 system_thread(void *data)
 
       _be_sys_get_executable_name(exe, sizeof(exe));
 
-      strncat(sig, get_filename(exe), sizeof(sig)-1);
+      strncat(sig, get_filename(exe), sizeof(sig));
       sig[sizeof(sig)-1] = '\0';
 
       _be_allegro_app = new BeAllegroApp(sig);
@@ -131,7 +133,7 @@ static int32 system_thread(void *data)
          p = &exe[strlen(exe) - 1];
          while (*p != '/') p--;
          *(p + 1) = '\0';
-         _al_sane_strncpy(app_path, exe, MAXPATHLEN);
+         strcpy(app_path, exe);
       }
    }
    else {
@@ -426,7 +428,7 @@ extern "C" void be_sys_get_gfx_safe_mode(int *driver, struct GFX_MODE *mode)
 
 extern "C" void be_sys_yield_timeslice(void)
 {
-   snooze(0);
+   snooze(YIELD_TIME);
 }
 
 
