@@ -186,8 +186,9 @@ static BITMAP *read_bitmap(PACKFILE *f, int bits, int allowconv)
 	       if (rgba)
 		  pack_getc(f);
 
-	       c = makecol24(r, g, b);
-	       WRITE3BYTES(bmp->line[y] + (x * 3), c);
+	       bmp->line[y][x*3+_rgb_r_shift_24/8] = r;
+	       bmp->line[y][x*3+_rgb_g_shift_24/8] = g;
+	       bmp->line[y][x*3+_rgb_b_shift_24/8] = b;
 	    }
 	 }
 	 break;
@@ -1181,11 +1182,11 @@ static int load_object(DATAFILE *obj, PACKFILE *f, int type)
 
 
 
-/* _load_property:
+/* load_property:
  *  Helper to load a property from a datafile and store it in 'prop'.
  *  Returns 0 on success and -1 on failure.
  */
-int _load_property(DATAFILE_PROPERTY *prop, PACKFILE *f)
+static int load_property(DATAFILE_PROPERTY *prop, PACKFILE *f)
 {
    int type, size;
    char *p;
@@ -1224,11 +1225,11 @@ int _load_property(DATAFILE_PROPERTY *prop, PACKFILE *f)
 
 
 
-/* _add_property:
+/* add_property:
  *  Helper to add a new property to a property list. Returns 0 on
  *  success or -1 on failure.
  */
-int _add_property(DATAFILE_PROPERTY **list, DATAFILE_PROPERTY *prop)
+static int add_property(DATAFILE_PROPERTY **list, DATAFILE_PROPERTY *prop)
 {
    DATAFILE_PROPERTY *iter;
    int length = 0;
@@ -1264,10 +1265,10 @@ int _add_property(DATAFILE_PROPERTY **list, DATAFILE_PROPERTY *prop)
 
 
 
-/* _destroy_property_list:
+/* destroy_property_list:
  *  Helper to destroy a property list.
  */
-void _destroy_property_list(DATAFILE_PROPERTY *list)
+static void destroy_property_list(DATAFILE_PROPERTY *list)
 {
    int c;
 
@@ -1306,7 +1307,7 @@ static void *load_file_object(PACKFILE *f, long size)
       type = pack_mgetl(f);
 
       if (type == DAT_PROPERTY) {
-	 if ((_load_property(&prop, f) != 0) || (_add_property(&list, &prop) != 0)) {
+	 if ((load_property(&prop, f) != 0) || (add_property(&list, &prop) != 0)) {
 	    failed = TRUE;
 	    break;
 	 }
@@ -1334,7 +1335,7 @@ static void *load_file_object(PACKFILE *f, long size)
 
    /* destroy the property list if not assigned to an object */
    if (list)
-      _destroy_property_list(list);
+      destroy_property_list(list);
 
    /* gracefully handle failure */
    if (failed) {
@@ -1464,7 +1465,7 @@ DATAFILE *load_datafile_object(AL_CONST char *filename, AL_CONST char *objectnam
       type = pack_mgetl(f);
 
       if (type == DAT_PROPERTY) {
-	 if ((_load_property(&prop, f) != 0) || (_add_property(&list, &prop) != 0))
+	 if ((load_property(&prop, f) != 0) || (add_property(&list, &prop) != 0))
 	    break;
 
 	 if ((prop.type == DAT_NAME) && (ustricmp(prop.dat, child) == 0))
@@ -1498,7 +1499,7 @@ DATAFILE *load_datafile_object(AL_CONST char *filename, AL_CONST char *objectnam
 
 	    /* destroy the property list */
 	    if (list) {
-	       _destroy_property_list(list);
+	       destroy_property_list(list);
 	       list = NULL;
 	    }
 
@@ -1509,7 +1510,7 @@ DATAFILE *load_datafile_object(AL_CONST char *filename, AL_CONST char *objectnam
 
    /* destroy the property list if not assigned to an object */
    if (list)
-      _destroy_property_list(list);
+      destroy_property_list(list);
 
    pack_fclose(f);
    return dat;
@@ -1526,7 +1527,7 @@ void _unload_datafile_object(DATAFILE *dat)
 
    /* destroy the property list */
    if (dat->prop)
-      _destroy_property_list(dat->prop);
+      destroy_property_list(dat->prop);
 
    /* look for a destructor function */
    for (i=0; i<MAX_DATAFILE_TYPES; i++) {
