@@ -597,19 +597,19 @@ char *append_filename(char *dest, AL_CONST char *path, AL_CONST char *filename,
  */
 char *get_filename(AL_CONST char *path)
 {
-   int c;
-   const char *ptr, *ret;
+   int pos, c;
    ASSERT(path);
 
-   ptr = path;
-   ret = ptr;
-   for (;;) {
-      c = ugetxc(&ptr);
-      if (!c) break;
+   pos = ustrlen(path);
+
+   while (pos>0) {
+      c = ugetat(path, pos-1);
       if ((c == '/') || (c == OTHER_PATH_SEPARATOR) || (c == DEVICE_SEPARATOR) || (c == '#'))
-         ret = (char*)ptr;
+	 break;
+      pos--;
    }
-   return (char*)ret;
+
+   return (char *)path + uoffset(path, pos);
 }
 
 
@@ -1398,7 +1398,7 @@ static int clone_password(PACKFILE *f)
 	 *allegro_errno = ENOMEM;
 	 return FALSE;
       }
-      _al_sane_strncpy(f->passdata, the_password, strlen(the_password)+1);
+      strcpy(f->passdata, the_password);
    }
    else
       f->passdata = NULL;
@@ -1852,7 +1852,7 @@ PACKFILE *pack_fopen_chunk(PACKFILE *f, int pack)
 	       free(chunk);
 	       return NULL;
 	    }
-	    _al_sane_strncpy(chunk->passdata, f->passdata, strlen(f->passdata)+1);
+	    strcpy(chunk->passdata, f->passdata);
 	    chunk->passpos = chunk->passdata + (long)f->passpos - (long)f->passdata;
 	    f->passpos = f->passdata;
 	 }
