@@ -21,7 +21,7 @@
 #include <string.h>
 
 #include "allegro.h"
-#include "allegro/aintern.h"
+#include "allegro/internal/aintern.h"
 
 
 #define FLI_MAGIC1            0xAF11      /* file header magic number */
@@ -330,7 +330,7 @@ static void do_fli_256_color(unsigned char *p, int sz)
       if (end > PAL_SIZE)
 	 return;
       else if ((sz -= length * 3) < 0) {
-	 FLI_KLUDGE(p, sz, length * 3);
+	 FLI_KLUDGE((char *)p, sz, length * 3);
       }
 
       fli_pal_dirty_from = MIN(fli_pal_dirty_from, offset);
@@ -396,7 +396,7 @@ static void do_fli_delta(unsigned char *p, int sz)
 	    if ((curr + size * 2) > bitmap_end)
 	       return;
 	    else if ((sz -= size * 2) < 0) {
-	       FLI_KLUDGE(p, sz, size * 2);
+	       FLI_KLUDGE((char *)p, sz, size * 2);
 	    }
 	    READ_BLOCK_NC(p, curr, size*2);
 	    curr += size*2;
@@ -406,7 +406,7 @@ static void do_fli_delta(unsigned char *p, int sz)
 	    if ((curr + size * 2) > bitmap_end)
 	       return;
 	    else if ((sz -= 2) < 0) {
-	       FLI_KLUDGE(p, sz, 2);
+	       FLI_KLUDGE((char *)p, sz, 2);
 	    }
 	    READ_RLE_WORD_NC(p, curr, size);
 	    curr += size*2;
@@ -446,7 +446,7 @@ static void do_fli_color(unsigned char *p, int sz)
       if (end > PAL_SIZE)
 	 return;
       else if ((sz -= length * 3) < 0) {
-	 FLI_KLUDGE(p, sz, length * 3);
+	 FLI_KLUDGE((char *)p, sz, length * 3);
       }
 
       fli_pal_dirty_from = MIN(fli_pal_dirty_from, offset);
@@ -503,7 +503,7 @@ static void do_fli_lc(unsigned char *p, int sz)
 	    if ((curr + size) > bitmap_end)
 	       return;
 	    else if ((sz -= size) < 0) {
-	       FLI_KLUDGE(p, sz, size);
+	       FLI_KLUDGE((char *)p, sz, size);
 	    }
 	    READ_BLOCK_NC(p, curr, size);
 	    curr += size;
@@ -513,7 +513,7 @@ static void do_fli_lc(unsigned char *p, int sz)
 	    if ((curr + size) > bitmap_end)
 	       return;
 	    else if ((sz -= 1) < 0) {
-	       FLI_KLUDGE(p, sz, 1);
+	       FLI_KLUDGE((char *)p, sz, 1);
 	    }
 	    READ_RLE_BYTE_NC(p, curr, size);
 	    curr += size;
@@ -529,9 +529,9 @@ static void do_fli_lc(unsigned char *p, int sz)
 /* do_fli_black:
  *  Processes an FLI BLACK chunk
  */
-static void do_fli_black()
+static void do_fli_black(void)
 {
-   clear(fli_bitmap);
+   clear_bitmap(fli_bitmap);
 
    fli_bmp_dirty_from = 0;
    fli_bmp_dirty_to = fli_bitmap->h-1;
@@ -572,7 +572,7 @@ static void do_fli_brun(unsigned char *p, int sz)
 	       if ((curr + size) > bitmap_end)
 		  return;
 	       else if ((sz -= size) < 0) {
-		  FLI_KLUDGE(p, sz, size);
+		  FLI_KLUDGE((char *)p, sz, size);
 	       }
 	       READ_BLOCK_NC(p, curr, size);
 	       curr += size;
@@ -581,7 +581,7 @@ static void do_fli_brun(unsigned char *p, int sz)
 	       if ((curr + size) > bitmap_end)
 		  return;
 	       else if ((sz -= 1) < 0) {
-		  FLI_KLUDGE(p, sz, 1);
+		  FLI_KLUDGE((char *)p, sz, 1);
 	       }
 	       READ_RLE_BYTE_NC(p, curr, size);
 	       curr += size;
@@ -600,7 +600,7 @@ static void do_fli_brun(unsigned char *p, int sz)
 	       if ((curr + size) > bitmap_end)
 		  return;
 	       if ((sz -= size) < 0) {
-		  FLI_KLUDGE(p, sz, size);
+		  FLI_KLUDGE((char *)p, sz, size);
 	       }
 	       READ_BLOCK_NC(p, curr, size);
 	       curr += size;
@@ -609,7 +609,7 @@ static void do_fli_brun(unsigned char *p, int sz)
 	       if ((curr + size) > bitmap_end)
 		  return;
 	       if ((sz -= 1) < 0) {
-		  FLI_KLUDGE(p, sz, 1);
+		  FLI_KLUDGE((char *)p, sz, 1);
 	       }
 	       READ_RLE_BYTE_NC(p, curr, size);
 	       curr += size;
@@ -704,7 +704,7 @@ static int _fli_parse_chunk(FLI_CHUNK *chunk, unsigned char *p, unsigned long fr
 /* read_frame:
  *  Advances to the next frame in the FLI.
  */
-static void read_frame()
+static void read_frame(void)
 {
    unsigned char *p;
    FLI_CHUNK chunk;
@@ -715,7 +715,7 @@ static void read_frame()
 
    /* clear the first frame (we need it for looping, because we don't support ring frame) */
    if (fli_frame == 0) {
-      clear(fli_bitmap);
+      clear_bitmap(fli_bitmap);
       fli_bmp_dirty_from = 0;
       fli_bmp_dirty_to = fli_bitmap->h-1;
    }
@@ -823,7 +823,7 @@ static void read_frame()
  *  reading a frame and displaying it, rather than between one frame and
  *  the next, in order to make the playback as smooth as possible.
  */
-static int do_play_fli(BITMAP *bmp, int loop, int (*callback)())
+static int do_play_fli(BITMAP *bmp, int loop, int (*callback)(void))
 {
    int ret;
 
@@ -875,8 +875,11 @@ static int do_play_fli(BITMAP *bmp, int loop, int (*callback)())
  *  callback should return positive integers, since the FLI status values 
  *  are zero or negative.
  */
-int play_fli(char *filename, BITMAP *bmp, int loop, int (*callback)(void))
+int play_fli(AL_CONST char *filename, BITMAP *bmp, int loop, int (*callback)(void))
 {
+   ASSERT(filename);
+   ASSERT(bmp);
+   
    if (open_fli(filename) != FLI_OK)
       return FLI_ERROR;
 
@@ -891,6 +894,9 @@ int play_fli(char *filename, BITMAP *bmp, int loop, int (*callback)(void))
  */
 int play_memory_fli(void *fli_data, BITMAP *bmp, int loop, int (*callback)(void))
 {
+   ASSERT(fli_data);
+   ASSERT(bmp);
+   
    if (open_memory_fli(fli_data) != FLI_OK)
       return FLI_ERROR;
 
@@ -902,7 +908,7 @@ int play_memory_fli(void *fli_data, BITMAP *bmp, int loop, int (*callback)(void)
 /* do_open_fli:
  *  Worker function used by open_fli() and open_memory_fli().
  */
-static int do_open_fli()
+static int do_open_fli(void)
 {
    long speed;
 
@@ -959,8 +965,10 @@ static int do_open_fli()
 /* open_fli:
  *  Opens an FLI file ready for playing.
  */
-int open_fli(char *filename)
+int open_fli(AL_CONST char *filename)
 {
+   ASSERT(filename);
+   
    if (fli_status != FLI_NOT_OPEN)
       return FLI_ERROR;
 
@@ -969,12 +977,10 @@ int open_fli(char *filename)
       fli_filename = NULL;
    }
 
-   fli_filename = malloc(ustrsizez(filename));
+   fli_filename = ustrdup(filename);
    if (!fli_filename)
       return FLI_ERROR;
 
-   ustrcpy(fli_filename, filename);
-
    fli_file = pack_fopen(fli_filename, F_READ);
    if (!fli_file)
       return FLI_ERROR;
@@ -990,6 +996,8 @@ int open_fli(char *filename)
  */
 int open_memory_fli(void *fli_data)
 {
+   ASSERT(fli_data);
+   
    if (fli_status != FLI_NOT_OPEN)
       return FLI_ERROR;
 
