@@ -25,21 +25,25 @@
 
 
 #include "allegro.h"
-#include "allegro/aintern.h"
+#include "allegro/internal/aintern.h"
 
 
+#ifdef ALLEGRO_WINDOWS
+/* exported address of d_clear_proc */
+int (*_d_clear_proc)(int, DIALOG *, int) = NULL;
+#endif
 
 /* typedef for the listbox callback functions */
 typedef char *(*getfuncptr)(int, int *);
 
 
 
-/* gui_textout:
+/* gui_textout_ex:
  *  Wrapper function for drawing text to the screen, which interprets the
  *  & character as an underbar for displaying keyboard shortcuts. Returns
  *  the width of the output string in pixels.
  */
-int gui_textout(BITMAP *bmp, char *s, int x, int y, int color, int centre)
+int gui_textout_ex(BITMAP *bmp, AL_CONST char *s, int x, int y, int color, int bg, int centre)
 {
    char tmp[1024];
    int hline_pos = -1;
@@ -47,6 +51,7 @@ int gui_textout(BITMAP *bmp, char *s, int x, int y, int color, int centre)
    int in_pos = 0;
    int out_pos = 0;
    int pix_len, c;
+   ASSERT(s);
 
    while (((c = ugetc(s+in_pos)) != 0) && (out_pos<(int)(sizeof(tmp)-ucwidth(0)))) {
       if (c == '&') {
@@ -74,7 +79,7 @@ int gui_textout(BITMAP *bmp, char *s, int x, int y, int color, int centre)
       x -= pix_len / 2;
 
    if (bmp) {
-      textout(bmp, font, tmp, x, y, color);
+      textout_ex(bmp, font, tmp, x, y, color, bg);
 
       if (hline_pos >= 0) {
 	 c = ugetat(tmp, hline_pos);
@@ -95,9 +100,10 @@ int gui_textout(BITMAP *bmp, char *s, int x, int y, int color, int centre)
 /* gui_strlen:
  *  Returns the length of a string in pixels, ignoring '&' characters.
  */
-int gui_strlen(char *s)
+int gui_strlen(AL_CONST char *s)
 {
-   return gui_textout(NULL, s, 0, 0, 0, 0);
+   ASSERT(s);
+   return gui_textout_ex(NULL, s, 0, 0, 0, 0, 0);
 }
 
 
@@ -110,10 +116,11 @@ static void dotted_rect(int x1, int y1, int x2, int y2, int fg, int bg)
    int x = ((x1+y1) & 1) ? 1 : 0;
    int c;
 
-   for (c=x1; c<=x2; c++) {
+   /* two loops to avoid bank switches */
+   for (c=x1; c<=x2; c++)
       putpixel(screen, c, y1, (((c+y1) & 1) == x) ? fg : bg);
+   for (c=x1; c<=x2; c++)
       putpixel(screen, c, y2, (((c+y2) & 1) == x) ? fg : bg);
-   }
 
    for (c=y1+1; c<y2; c++) {
       putpixel(screen, x1, c, (((c+x1) & 1) == x) ? fg : bg);
@@ -123,12 +130,34 @@ static void dotted_rect(int x1, int y1, int x2, int y2, int fg, int bg)
 
 
 
+/* d_yield_proc:
+ *  Simple dialog procedure which just yields the timeslice when the dialog
+ *  is idle.
+ */
+int d_yield_proc(int msg, DIALOG *d, int c)
+{
+   if (msg == MSG_IDLE)
+      yield_timeslice();
+
+   return D_O_K;
+}
+
+
+
 /* d_clear_proc:
  *  Simple dialog procedure which just clears the screen. Useful as the
  *  first object in a dialog.
  */
 int d_clear_proc(int msg, DIALOG *d, int c)
 {
+   ASSERT(d);
+
+#ifdef ALLEGRO_WINDOWS
+   /* kludge to get the exported address of d_clear_proc */
+   if (!_d_clear_proc)
+      _d_clear_proc = d->proc;
+#endif
+
    if (msg == MSG_DRAW) {
       set_clip(screen, 0, 0, SCREEN_W-1, SCREEN_H-1);
       clear_to_color(screen, d->bg);
@@ -144,10 +173,11 @@ int d_clear_proc(int msg, DIALOG *d, int c)
  */
 int d_box_proc(int msg, DIALOG *d, int c)
 {
+   ASSERT(d);
    if (msg==MSG_DRAW) {
       int fg = (d->flags & D_DISABLED) ? gui_mg_color : d->fg;
-      rectfill(screen, d->x+1, d->y+1, d->x+d->w-1, d->y+d->h-1, d->bg);
-      rect(screen, d->x, d->y, d->x+d->w, d->y+d->h, fg);
+      rectfill(screen, d->x+1, d->y+1, d->x+d->w-2, d->y+d->h-2, d->bg);
+      rect(screen, d->x, d->y, d->x+d->w-1, d->y+d->h-1, fg);
    }
 
    return D_O_K;
@@ -160,12 +190,14 @@ int d_box_proc(int msg, DIALOG *d, int c)
  */
 int d_shadow_box_proc(int msg, DIALOG *d, int c)
 {
+   ASSERT(d);
    if (msg==MSG_DRAW) {
       int fg = (d->flags & D_DISABLED) ? gui_mg_color : d->fg;
-      rectfill(screen, d->x+1, d->y+1, d->x+d->w-2, d->y+d->h-2, d->bg);
-      rect(screen, d->x, d->y, d->x+d->w-1, d->y+d->h-1, fg);
-      vline(screen, d->x+d->w, d->y+1, d->y+d->h, fg);
-      hline(screen, d->x+1, d->y+d->h, d->x+d->w, fg);
+      int black = makecol(0,0,0);
+      rectfill(screen, d->x+1, d->y+1, d->x+d->w-3, d->y+d->h-3, d->bg);
+      rect(screen, d->x, d->y, d->x+d->w-2, d->y+d->h-2, fg);
+      vline(screen, d->x+d->w-1, d->y+1, d->y+d->h-1, black);
+      hline(screen, d->x+1, d->y+d->h-1, d->x+d->w-1, black);
    }
 
    return D_O_K;
@@ -178,8 +210,10 @@ int d_shadow_box_proc(int msg, DIALOG *d, int c)
  */
 int d_bitmap_proc(int msg, DIALOG *d, int c)
 {
-   BITMAP *b = (BITMAP *)d->dp;
+   BITMAP *b;
+   ASSERT(d);
 
+   b = (BITMAP *)d->dp;
    if (msg==MSG_DRAW)
       blit(b, screen, 0, 0, d->x, d->y, d->w, d->h);
 
@@ -193,6 +227,7 @@ int d_bitmap_proc(int msg, DIALOG *d, int c)
  */
 int d_text_proc(int msg, DIALOG *d, int c)
 {
+   ASSERT(d);
    if (msg==MSG_DRAW) {
       int fg = (d->flags & D_DISABLED) ? gui_mg_color : d->fg;
       FONT *oldfont = font;
@@ -200,8 +235,7 @@ int d_text_proc(int msg, DIALOG *d, int c)
       if (d->dp2)
 	 font = d->dp2;
 
-      text_mode(d->bg);
-      gui_textout(screen, d->dp, d->x, d->y, fg, FALSE);
+      gui_textout_ex(screen, d->dp, d->x, d->y, fg, d->bg, FALSE);
 
       font = oldfont;
    }
@@ -217,6 +251,7 @@ int d_text_proc(int msg, DIALOG *d, int c)
  */
 int d_ctext_proc(int msg, DIALOG *d, int c)
 {
+   ASSERT(d);
    if (msg==MSG_DRAW) {
       int fg = (d->flags & D_DISABLED) ? gui_mg_color : d->fg;
       FONT *oldfont = font;
@@ -224,8 +259,31 @@ int d_ctext_proc(int msg, DIALOG *d, int c)
       if (d->dp2)
 	 font = d->dp2;
 
-      text_mode(d->bg);
-      gui_textout(screen, d->dp, d->x, d->y, fg, TRUE);
+      gui_textout_ex(screen, d->dp, d->x + d->w/2, d->y, fg, d->bg, TRUE);
+
+      font = oldfont;
+   }
+
+   return D_O_K;
+}
+
+
+
+/* d_rtext_proc:
+ *  Simple dialog procedure: draws the text string which is pointed to by dp,
+ *  right aligning it.
+ */
+int d_rtext_proc(int msg, DIALOG *d, int c)
+{
+   ASSERT(d);
+   if (msg==MSG_DRAW) {
+      int fg = (d->flags & D_DISABLED) ? gui_mg_color : d->fg;
+      FONT *oldfont = font;
+
+      if (d->dp2)
+	 font = d->dp2;
+
+      gui_textout_ex(screen, d->dp, d->x + d->w - gui_strlen(d->dp), d->y, fg, d->bg, FALSE);
 
       font = oldfont;
    }
@@ -244,8 +302,10 @@ int d_ctext_proc(int msg, DIALOG *d, int c)
 int d_button_proc(int msg, DIALOG *d, int c)
 {
    int state1, state2;
+   int black;
    int swap;
    int g;
+   ASSERT(d);
 
    switch (msg) {
 
@@ -256,27 +316,27 @@ int d_button_proc(int msg, DIALOG *d, int c)
 	    state2 = (d->flags & D_DISABLED) ? gui_mg_color : d->fg;
 	 }
 	 else {
-	    g = 0; 
+	    g = 0;
 	    state1 = (d->flags & D_DISABLED) ? gui_mg_color : d->fg;
 	    state2 = d->bg;
 	 }
 
-	 rectfill(screen, d->x+1+g, d->y+1+g, d->x+d->w-2+g, d->y+d->h-2+g, state2);
-	 rect(screen, d->x+g, d->y+g, d->x+d->w-1+g, d->y+d->h-1+g, state1);
-	 text_mode(-1);
-	 gui_textout(screen, d->dp, d->x+d->w/2+g, d->y+d->h/2-text_height(font)/2+g, state1, TRUE);
+	 rectfill(screen, d->x+1+g, d->y+1+g, d->x+d->w-3+g, d->y+d->h-3+g, state2);
+	 rect(screen, d->x+g, d->y+g, d->x+d->w-2+g, d->y+d->h-2+g, state1);
+	 gui_textout_ex(screen, d->dp, d->x+d->w/2+g, d->y+d->h/2-text_height(font)/2+g, state1, -1, TRUE);
 
 	 if (d->flags & D_SELECTED) {
-	    vline(screen, d->x, d->y, d->y+d->h-1, d->bg);
-	    hline(screen, d->x, d->y, d->x+d->w-1, d->bg);
+	    vline(screen, d->x, d->y, d->y+d->h-2, d->bg);
+	    hline(screen, d->x, d->y, d->x+d->w-2, d->bg);
 	 }
 	 else {
-	    vline(screen, d->x+d->w, d->y+1, d->y+d->h-1, d->fg);
-	    hline(screen, d->x+1, d->y+d->h, d->x+d->w, d->fg);
+	    black = makecol(0,0,0);
+	    vline(screen, d->x+d->w-1, d->y+1, d->y+d->h-2, black);
+	    hline(screen, d->x+1, d->y+d->h-1, d->x+d->w-1, black);
 	 }
 	 if ((d->flags & D_GOTFOCUS) && 
 	     (!(d->flags & D_SELECTED) || !(d->flags & D_EXIT)))
-	    dotted_rect(d->x+1+g, d->y+1+g, d->x+d->w-2+g, d->y+d->h-2+g, state1, state2);
+	    dotted_rect(d->x+1+g, d->y+1+g, d->x+d->w-3+g, d->y+d->h-3+g, state1, state2);
 	 break;
 
       case MSG_WANTFOCUS:
@@ -290,9 +350,7 @@ int d_button_proc(int msg, DIALOG *d, int c)
 
 	 /* or just toggle */
 	 d->flags ^= D_SELECTED;
-	 scare_mouse();
-	 SEND_MESSAGE(d, MSG_DRAW, 0);
-	 unscare_mouse();
+	 object_message(d, MSG_DRAW, 0);
 	 break;
 
       case MSG_CLICK:
@@ -314,9 +372,7 @@ int d_button_proc(int msg, DIALOG *d, int c)
 	    if (((state1) && (!state2)) || ((state2) && (!state1))) {
 	       d->flags ^= D_SELECTED;
 	       state1 = d->flags & D_SELECTED;
-	       scare_mouse();
-	       SEND_MESSAGE(d, MSG_DRAW, 0);
-	       unscare_mouse();
+	       object_message(d, MSG_DRAW, 0);
 	    }
 
 	    /* let other objects continue to animate */
@@ -344,22 +400,24 @@ int d_check_proc(int msg, DIALOG *d, int c)
 {
    int x;
    int fg, bg;
+   ASSERT(d);
 
    if (msg==MSG_DRAW) {
       fg = (d->flags & D_DISABLED) ? gui_mg_color : d->fg;
       bg = (d->bg < 0) ? gui_bg_color : d->bg;
-      text_mode(d->bg);
-      x = d->x + ((d->d1) ? 0 : gui_textout(screen, d->dp, d->x, d->y+(d->h-(text_height(font)-gui_font_baseline))/2, fg, FALSE) + text_height(font)/2);
-      rectfill(screen, x+1, d->y+1, x+d->h-1, d->y+d->h-1, bg);
-      rect(screen, x, d->y, x+d->h, d->y+d->h, fg);
+
+      x = d->x + ((d->d1) ? 0 : gui_textout_ex(screen, d->dp, d->x, d->y+(d->h-(text_height(font)-gui_font_baseline))/2, fg, d->bg, FALSE) + text_height(font)/2);
+      rectfill(screen, x+1, d->y+1, x+d->h-2, d->y+d->h-2, bg);
+      rect(screen, x, d->y, x+d->h-1, d->y+d->h-1, fg);
       if (d->d1)
-	 gui_textout(screen, d->dp, x+d->h+text_height(font)/2, d->y+(d->h-(text_height(font)-gui_font_baseline))/2, fg, FALSE);
+	 gui_textout_ex(screen, d->dp, x+d->h-1+text_height(font)/2, d->y+(d->h-(text_height(font)-gui_font_baseline))/2, fg, d->bg, FALSE);
       if (d->flags & D_SELECTED) {
-	 line(screen, x, d->y, x+d->h, d->y+d->h, fg);
-	 line(screen, x, d->y+d->h, x+d->h, d->y, fg); 
+	 line(screen, x, d->y, x+d->h-1, d->y+d->h-1, fg);
+	 line(screen, x, d->y+d->h-1, x+d->h-1, d->y, fg);
       }
       if (d->flags & D_GOTFOCUS)
-	 dotted_rect(x+1, d->y+1, x+d->h-1, d->y+d->h-1, fg, bg);
+	 dotted_rect(x+1, d->y+1, x+d->h-2, d->y+d->h-2, fg, bg);
+
       return D_O_K;
    } 
 
@@ -376,27 +434,28 @@ int d_check_proc(int msg, DIALOG *d, int c)
 int d_radio_proc(int msg, DIALOG *d, int c)
 {
    int x, center, r, ret, fg, bg;
+   ASSERT(d);
 
    switch(msg) {
 
       case MSG_DRAW:
 	 fg = (d->flags & D_DISABLED) ? gui_mg_color : d->fg;
 	 bg = (d->bg < 0) ? gui_bg_color : d->bg;
-	 text_mode(d->bg);
-	 gui_textout(screen, d->dp, d->x+d->h+text_height(font), d->y+(d->h-(text_height(font)-gui_font_baseline))/2, fg, FALSE);
+
+	 gui_textout_ex(screen, d->dp, d->x+d->h-1+text_height(font), d->y+(d->h-(text_height(font)-gui_font_baseline))/2, fg, d->bg, FALSE);
 
 	 x = d->x;
 	 r = d->h/2;
 
 	 center = x+r;
-	 rectfill(screen, x+1, d->y+1, x+d->h-1, d->y+d->h-1, bg);
+	 rectfill(screen, x, d->y, x+d->h-1, d->y+d->h-1, bg);
 
 	 switch (d->d2) {
 
 	    case 1:
-	       rect(screen, x, d->y, x+d->h, d->y+d->h, fg);
+	       rect(screen, x, d->y, x+d->h-1, d->y+d->h-1, fg);
 	       if (d->flags & D_SELECTED)
-		  rectfill(screen, x+r/2, d->y+r/2, x+d->h-r/2, d->y+d->h-r/2, fg);
+		  rectfill(screen, x+r/2, d->y+r/2, x+d->h-1-r/2, d->y+d->h-1-r/2, fg);
 	       break;
 
 	    default:
@@ -406,8 +465,12 @@ int d_radio_proc(int msg, DIALOG *d, int c)
 	       break;
 	 }
 
-	 if (d->flags & D_GOTFOCUS)
-	    dotted_rect(x+1, d->y+1, x+d->h-1, d->y+d->h-1, fg, bg);
+	 if (d->flags & D_GOTFOCUS) {
+	    if (d->d2 == 1)
+	       dotted_rect(x+1, d->y+1, x+d->h-2, d->y+d->h-2, fg, bg);
+	    else
+	       dotted_rect(x, d->y, x+d->h-1, d->y+d->h-1, fg, bg);
+	 }
 
 	 return D_O_K;
 
@@ -421,9 +484,7 @@ int d_radio_proc(int msg, DIALOG *d, int c)
       case MSG_RADIO:
 	 if ((c == d->d1) && (d->flags & D_SELECTED)) {
 	    d->flags &= ~D_SELECTED;
-	    scare_mouse();
-	    SEND_MESSAGE(d, MSG_DRAW, 0);
-	    unscare_mouse();
+	    object_message(d, MSG_DRAW, 0);
 	 }
 	 break;
    }
@@ -459,13 +520,15 @@ int d_radio_proc(int msg, DIALOG *d, int c)
  */
 int d_icon_proc(int msg, DIALOG *d, int c)
 {
-   BITMAP *butimage = (BITMAP *)d->dp;
+   BITMAP *butimage;
    int butx;
    int buty;
    int index;
    int indent;
    int depth;
+   ASSERT(d);
 
+   butimage = (BITMAP *)d->dp;
    if ((msg == MSG_DRAW) && (!(d->flags & D_HIDDEN))) {
       depth = 0;
       if ((d->dp2 == NULL) && (d->flags & D_SELECTED)) {
@@ -525,8 +588,9 @@ int d_icon_proc(int msg, DIALOG *d, int c)
  */
 int d_keyboard_proc(int msg, DIALOG *d, int c)
 {
-   int (*proc)();
+   int (*proc)(void);
    int ret = D_O_K;
+   ASSERT(d);
 
    switch (msg) {
 
@@ -562,9 +626,11 @@ int d_keyboard_proc(int msg, DIALOG *d, int c)
 int d_edit_proc(int msg, DIALOG *d, int c)
 {
    static int ignore_next_uchar = FALSE;
+   int last_was_space, new_pos, i, k;
    int f, l, p, w, x, fg, b, scroll;
    char buf[16];
-   char *s;
+   char *s, *t;
+   ASSERT(d);
 
    s = d->dp;
    l = ustrlen(s);
@@ -619,11 +685,10 @@ int d_edit_proc(int msg, DIALOG *d, int c)
 	    f = ugetat(s, p);
 	    usetc(buf+usetc(buf, (f) ? f : ' '), 0);
 	    w = text_length(font, buf);
-	    if (x+w > d->w) 
+	    if (x+w > d->w)
 	       break;
 	    f = ((p == d->d2) && (d->flags & D_GOTFOCUS));
-	    text_mode((f) ? fg : d->bg);
-	    textout(screen, font, buf, d->x+x, d->y, (f) ? d->bg : fg);
+	    textout_ex(screen, font, buf, d->x+x, d->y, (f) ? d->bg : fg, (f) ? fg : d->bg);
 	    x += w;
 	 }
 	 if (x < d->w)
@@ -647,9 +712,7 @@ int d_edit_proc(int msg, DIALOG *d, int c)
 	       break;
 	 }
 	 d->d2 = MID(0, p, l);
-	 scare_mouse();
-	 SEND_MESSAGE(d, MSG_DRAW, 0);
-	 unscare_mouse();
+	 object_message(d, MSG_DRAW, 0);
 	 break;
 
       case MSG_WANTFOCUS:
@@ -661,10 +724,38 @@ int d_edit_proc(int msg, DIALOG *d, int c)
 	 ignore_next_uchar = FALSE;
 
 	 if ((c >> 8) == KEY_LEFT) {
-	    if (d->d2 > 0) d->d2--;
+	    if (d->d2 > 0) {
+	       if (key_shifts & KB_CTRL_FLAG) {
+		  last_was_space = TRUE;
+		  new_pos = 0;
+		  t = s;
+		  for (i = 0; i < d->d2; i++) {
+		     k = ugetx(&t);
+		     if (uisspace(k))
+			last_was_space = TRUE;
+		     else if (last_was_space) {
+			last_was_space = FALSE;
+			new_pos = i;
+		     }
+		  }
+		  d->d2 = new_pos;
+	       }
+	       else
+		  d->d2--;
+	    }
 	 }
 	 else if ((c >> 8) == KEY_RIGHT) {
-	    if (d->d2 < l) d->d2++;
+	    if (d->d2 < l) {
+	       if (key_shifts & KB_CTRL_FLAG) {
+		  t = s + uoffset(s, d->d2);
+		  for (k = ugetx(&t); uisspace(k); k = ugetx(&t))
+		     d->d2++;
+		  for (; k && !uisspace(k); k = ugetx(&t))
+		     d->d2++;
+	       }
+	       else
+		  d->d2++;
+	    }
 	 }
 	 else if ((c >> 8) == KEY_HOME) {
 	    d->d2 = 0;
@@ -684,9 +775,7 @@ int d_edit_proc(int msg, DIALOG *d, int c)
 	 }
 	 else if ((c >> 8) == KEY_ENTER) {
 	    if (d->flags & D_EXIT) {
-	       scare_mouse();
-	       SEND_MESSAGE(d, MSG_DRAW, 0);
-	       unscare_mouse();
+	       object_message(d, MSG_DRAW, 0);
 	       return D_CLOSE;
 	    }
 	    else
@@ -700,9 +789,7 @@ int d_edit_proc(int msg, DIALOG *d, int c)
 	    /* don't process regular keys here: MSG_UCHAR will do that */
 	    break;
 	 }
-	 scare_mouse();
-	 SEND_MESSAGE(d, MSG_DRAW, 0);
-	 unscare_mouse();
+	 object_message(d, MSG_DRAW, 0);
 	 return D_USED_CHAR;
 
       case MSG_UCHAR:
@@ -711,9 +798,7 @@ int d_edit_proc(int msg, DIALOG *d, int c)
 	       uinsert(s, d->d2, c);
 	       d->d2++;
 
-	       scare_mouse();
-	       SEND_MESSAGE(d, MSG_DRAW, 0);
-	       unscare_mouse();
+	       object_message(d, MSG_DRAW, 0);
 	    }
 	    return D_USED_CHAR;
 	 }
@@ -728,11 +813,10 @@ int d_edit_proc(int msg, DIALOG *d, int c)
 /* _handle_scrollable_click:
  *  Helper to process a click on a scrollable object.
  */
-void _handle_scrollable_scroll_click(DIALOG *d, int listsize, int *offset)
+void _handle_scrollable_scroll_click(DIALOG *d, int listsize, int *offset, int height)
 {
    int xx, yy;
-   int height = (d->h-3) / text_height(font);
-   int hh = d->h-4;
+   int hh = d->h - 5;
 
    while (gui_mouse_b()) {
       int i = (hh * height + listsize/2) / listsize;
@@ -750,9 +834,7 @@ void _handle_scrollable_scroll_click(DIALOG *d, int listsize, int *offset)
 
 	    if (yy != *offset) {
 	       *offset = yy;
-	       scare_mouse();
-	       SEND_MESSAGE(d, MSG_DRAW, 0);
-	       unscare_mouse();
+	       object_message(d, MSG_DRAW, 0);
 	    }
 
 	    /* let other objects continue to animate */
@@ -773,9 +855,7 @@ void _handle_scrollable_scroll_click(DIALOG *d, int listsize, int *offset)
 
 	 if (yy != *offset) {
 	    *offset = yy;
-	    scare_mouse();
-	    SEND_MESSAGE(d, MSG_DRAW, 0);
-	    unscare_mouse();
+	    object_message(d, MSG_DRAW, 0);
 	 }
       }
 
@@ -791,7 +871,7 @@ void _handle_scrollable_scroll_click(DIALOG *d, int listsize, int *offset)
  */
 void _handle_scrollable_scroll(DIALOG *d, int listsize, int *index, int *offset)
 {
-   int height = (d->h-3) / text_height(font);
+   int height = (d->h-4) / text_height(font);
 
    if (listsize <= 0) {
       *index = *offset = 0;
@@ -846,10 +926,10 @@ void _handle_listbox_click(DIALOG *d)
    if (!listsize)
       return;
 
-   height = (d->h-3) / text_height(font);
+   height = (d->h-4) / text_height(font);
 
    i = MID(0, ((gui_mouse_y() - d->y - 2) / text_height(font)), 
-	      ((d->h-3) / text_height(font) - 1));
+	      ((d->h-4) / text_height(font) - 1));
    i += d->d2;
    if (i < d->d2)
       i = d->d2;
@@ -862,7 +942,7 @@ void _handle_listbox_click(DIALOG *d)
 
    if (gui_mouse_y() <= d->y)
       i = MAX(i-1, 0);
-   else if (gui_mouse_y() >= d->y+d->h)
+   else if (gui_mouse_y() >= d->y+d->h-1)
       i = MIN(i+1, listsize-1);
 
    if (i != d->d1) {
@@ -881,12 +961,10 @@ void _handle_listbox_click(DIALOG *d)
       i = d->d2;
       _handle_scrollable_scroll(d, listsize, &d->d1, &d->d2);
 
-      scare_mouse();
-      SEND_MESSAGE(d, MSG_DRAW, 0);
-      unscare_mouse();
+      d->flags |= D_DIRTY;
 
       if (i != d->d2)
-	 rest_callback(MID(10, text_height(font)*16-d->h, 100), idle_cb);
+	 rest_callback(MID(10, text_height(font)*16-d->h-1, 100), idle_cb);
    }
 }
 
@@ -902,26 +980,26 @@ void _draw_scrollable_frame(DIALOG *d, int listsize, int offset, int height, int
    int xx, yy;
 
    /* draw frame */
-   rect(screen, d->x, d->y, d->x+d->w, d->y+d->h, fg_color);
+   rect(screen, d->x, d->y, d->x+d->w-1, d->y+d->h-1, fg_color);
 
    /* possibly draw scrollbar */
    if (listsize > height) {
-      vline(screen, d->x+d->w-12, d->y+1, d->y+d->h-1, fg_color);
+      vline(screen, d->x+d->w-13, d->y+1, d->y+d->h-2, fg_color);
 
       /* scrollbar with focus */ 
       if (d->flags & D_GOTFOCUS) {
-	 dotted_rect(d->x+1, d->y+1, d->x+d->w-13, d->y+d->h-1, fg_color, bg);
-	 dotted_rect(d->x+d->w-11, d->y+1, d->x+d->w-1, d->y+d->h-1, fg_color, bg);
+	 dotted_rect(d->x+1, d->y+1, d->x+d->w-14, d->y+d->h-2, fg_color, bg);
+	 dotted_rect(d->x+d->w-12, d->y+1, d->x+d->w-2, d->y+d->h-2, fg_color, bg);
       }
       else {
-	 rect(screen, d->x+1, d->y+1, d->x+d->w-13, d->y+d->h-1, bg);
-	 rect(screen, d->x+d->w-11, d->y+1, d->x+d->w-1, d->y+d->h-1, bg);
+	 rect(screen, d->x+1, d->y+1, d->x+d->w-14, d->y+d->h-2, bg);
+	 rect(screen, d->x+d->w-12, d->y+1, d->x+d->w-2, d->y+d->h-2, bg);
       }
 
       /* create and draw the scrollbar */
       pattern = create_bitmap(2, 2);
-      i = ((d->h-4) * height + listsize/2) / listsize;
-      xx = d->x+d->w-10;
+      i = ((d->h-5) * height + listsize/2) / listsize;
+      xx = d->x+d->w-11;
       yy = d->y+2;
 
       putpixel(pattern, 0, 1, bg);
@@ -930,20 +1008,20 @@ void _draw_scrollable_frame(DIALOG *d, int listsize, int offset, int height, int
       putpixel(pattern, 1, 1, fg_color);
 
       if (offset > 0) {
-	 len = (((d->h-4) * offset) + listsize/2) / listsize;
-	 rectfill(screen, xx, yy, xx+8, yy+len-1, bg);
+	 len = (((d->h-5) * offset) + listsize/2) / listsize;
+	 rectfill(screen, xx, yy, xx+8, yy+len, bg);
 	 yy += len;
       }
-      if (yy+i < d->y+d->h-2) {
+      if (yy+i < d->y+d->h-3) {
 	 drawing_mode(DRAW_MODE_COPY_PATTERN, pattern, 0, 0);
 	 rectfill(screen, xx, yy, xx+8, yy+i, 0);
 	 solid_mode();
-	 yy += i;
-	 rectfill(screen, xx, yy, xx+8, d->y+d->h-2, bg);
+	 yy += i+1;
+	 rectfill(screen, xx, yy, xx+8, d->y+d->h-3, bg);
       }
       else {
 	 drawing_mode(DRAW_MODE_COPY_PATTERN, pattern, 0, 0);
-	 rectfill(screen, xx, yy, xx+8, d->y+d->h-2, 0);
+	 rectfill(screen, xx, yy, xx+8, d->y+d->h-3, 0);
 	 solid_mode();
       }
       destroy_bitmap(pattern);
@@ -951,9 +1029,9 @@ void _draw_scrollable_frame(DIALOG *d, int listsize, int offset, int height, int
    else {
       /* no scrollbar necessary */
       if (d->flags & D_GOTFOCUS)
-	 dotted_rect(d->x+1, d->y+1, d->x+d->w-1, d->y+d->h-1, fg_color, bg);
+	 dotted_rect(d->x+1, d->y+1, d->x+d->w-2, d->y+d->h-2, fg_color, bg);
       else
-	 rect(screen, d->x+1, d->y+1, d->x+d->w-1, d->y+d->h-1, bg);
+	 rect(screen, d->x+1, d->y+1, d->x+d->w-2, d->y+d->h-2, bg);
    }
 }
 
@@ -970,9 +1048,9 @@ void _draw_listbox(DIALOG *d)
    char s[1024];
 
    (*(getfuncptr)d->dp)(-1, &listsize);
-   height = (d->h-3) / text_height(font);
+   height = (d->h-4) / text_height(font);
    bar = (listsize > height);
-   w = (bar ? d->w-14 : d->w-2);
+   w = (bar ? d->w-15 : d->w-3);
    fg_color = (d->flags & D_DISABLED) ? gui_mg_color : d->fg;
 
    /* draw box contents */
@@ -990,19 +1068,17 @@ void _draw_listbox(DIALOG *d)
 	    fg = fg_color;
 	    bg = d->bg;
 	 }
-	 usetc(s, 0);
-	 ustrncat(s, (*(getfuncptr)d->dp)(i+d->d2, NULL), sizeof(s)-ucwidth(0));
+	 ustrzcpy(s, sizeof(s), (*(getfuncptr)d->dp)(i+d->d2, NULL));
 	 x = d->x + 2;
 	 y = d->y + 2 + i*text_height(font);
-	 text_mode(bg);
-	 rectfill(screen, x, y, x+7, y+text_height(font)-1, bg); 
+	 rectfill(screen, x, y, x+7, y+text_height(font)-1, bg);
 	 x += 8;
 	 len = ustrlen(s);
-	 while (text_length(font, s) >= MAX(d->w - (bar ? 22 : 10), 1)) {
+	 while (text_length(font, s) >= MAX(d->w - 1 - (bar ? 22 : 10), 1)) {
 	    len--;
 	    usetat(s, len, 0);
 	 }
-	 textout(screen, font, s, x, y, fg); 
+	 textout_ex(screen, font, s, x, y, fg, bg);
 	 x += text_length(font, s);
 	 if (x <= d->x+w) 
 	    rectfill(screen, x, y, d->x+w, y+text_height(font)-1, bg);
@@ -1013,9 +1089,9 @@ void _draw_listbox(DIALOG *d)
       }
    }
 
-   if (d->y+2+i*text_height(font) <= d->y+d->h-2)
+   if (d->y+2+i*text_height(font) <= d->y+d->h-3)
       rectfill(screen, d->x+2, d->y+2+i*text_height(font), 
-				       d->x+w, d->y+d->h-2, d->bg);
+				       d->x+w, d->y+d->h-3, d->bg);
 
    /* draw frame, maybe with scrollbar */
    _draw_scrollable_frame(d, listsize, d->d2, height, fg_color, d->bg);
@@ -1026,7 +1102,7 @@ void _draw_listbox(DIALOG *d)
 /* d_list_proc:
  *  A list box object. The dp field points to a function which it will call
  *  to obtain information about the list. This should follow the form:
- *     char *<list_func_name> (int index, int *list_size);
+ *     const char *<list_func_name> (int index, int *list_size);
  *  If index is zero or positive, the function should return a pointer to
  *  the string which is to be displayed at position index in the list. If
  *  index is  negative, it should return null and list_size should be set
@@ -1042,6 +1118,7 @@ int d_list_proc(int msg, DIALOG *d, int c)
    int listsize, i, bottom, height, bar, orig;
    char *sel = d->dp2;
    int redraw = FALSE;
+   ASSERT(d);
 
    switch (msg) {
 
@@ -1056,9 +1133,9 @@ int d_list_proc(int msg, DIALOG *d, int c)
 
       case MSG_CLICK:
 	 (*(getfuncptr)d->dp)(-1, &listsize);
-	 height = (d->h-3) / text_height(font);
+	 height = (d->h-4) / text_height(font);
 	 bar = (listsize > height);
-	 if ((!bar) || (gui_mouse_x() < d->x+d->w-10)) {
+	 if ((!bar) || (gui_mouse_x() < d->x+d->w-13)) {
 	    if ((sel) && (!(key_shifts & KB_CTRL_FLAG))) {
 	       for (i=0; i<listsize; i++) {
 		  if (sel[i]) {
@@ -1066,11 +1143,8 @@ int d_list_proc(int msg, DIALOG *d, int c)
 		     sel[i] = FALSE;
 		  }
 	       }
-	       if (redraw) {
-		  scare_mouse();
-		  SEND_MESSAGE(d, MSG_DRAW, 0);
-		  unscare_mouse();
-	       }
+	       if (redraw)
+		  object_message(d, MSG_DRAW, 0);
 	    }
 	    _handle_listbox_click(d);
 	    while (gui_mouse_b()) {
@@ -1081,19 +1155,19 @@ int d_list_proc(int msg, DIALOG *d, int c)
 	    }
 	 }
 	 else {
-	    _handle_scrollable_scroll_click(d, listsize, &d->d2);
+	    _handle_scrollable_scroll_click(d, listsize, &d->d2, height);
 	 }
 	 break;
 
       case MSG_DCLICK:
 	 (*(getfuncptr)d->dp)(-1, &listsize);
-	 height = (d->h-3) / text_height(font);
+	 height = (d->h-4) / text_height(font);
 	 bar = (listsize > height);
-	 if ((!bar) || (gui_mouse_x() < d->x+d->w-10)) {
+	 if ((!bar) || (gui_mouse_x() < d->x+d->w-13)) {
 	    if (d->flags & D_EXIT) {
 	       if (listsize) {
 		  i = d->d1;
-		  SEND_MESSAGE(d, MSG_CLICK, 0);
+		  object_message(d, MSG_CLICK, 0);
 		  if (i == d->d1) 
 		     return D_CLOSE;
 	       }
@@ -1103,17 +1177,16 @@ int d_list_proc(int msg, DIALOG *d, int c)
 
       case MSG_WHEEL:
 	 (*(getfuncptr)d->dp)(-1, &listsize);
-	 height = (d->h-3) / text_height(font);
+	 height = (d->h-4) / text_height(font);
 	 if (height < listsize) {
+	    int delta = (height > 3) ? 3 : 1;
 	    if (c > 0) 
-	       i = MAX(0, d->d2-3);
+	       i = MAX(0, d->d2-delta);
 	    else
-	       i = MIN(listsize-height, d->d2+3);
+	       i = MIN(listsize-height, d->d2+delta);
 	    if (i != d->d2) {
 	       d->d2 = i;
-	       scare_mouse();
-	       SEND_MESSAGE(d, MSG_DRAW, 0);
-	       unscare_mouse(); 
+	       object_message(d, MSG_DRAW, 0);
 	    }
 	 }
 	 break;
@@ -1133,7 +1206,7 @@ int d_list_proc(int msg, DIALOG *d, int c)
 	 if (listsize) {
 	    c >>= 8;
 
-	    bottom = d->d2 + (d->h-3)/text_height(font) - 1;
+	    bottom = d->d2 + (d->h-4)/text_height(font) - 1;
 	    if (bottom >= listsize-1)
 	       bottom = listsize-1;
 
@@ -1151,13 +1224,13 @@ int d_list_proc(int msg, DIALOG *d, int c)
 	       if (d->d1 > d->d2)
 		  d->d1 = d->d2;
 	       else
-		  d->d1 -= (bottom - d->d2);
+		  d->d1 -= (bottom - d->d2) ? bottom - d->d2 : 1;
 	    }
 	    else if (c == KEY_PGDN) {
 	       if (d->d1 < bottom)
 		  d->d1 = bottom;
 	       else
-		  d->d1 += (bottom - d->d2);
+		  d->d1 += (bottom - d->d2) ? bottom - d->d2 : 1;
 	    } 
 	    else 
 	       return D_O_K;
@@ -1179,9 +1252,7 @@ int d_list_proc(int msg, DIALOG *d, int c)
 
 	    /* if we changed something, better redraw... */ 
 	    _handle_scrollable_scroll(d, listsize, &d->d1, &d->d2);
-	    scare_mouse();
-	    SEND_MESSAGE(d, MSG_DRAW, 0);
-	    unscare_mouse(); 
+	    d->flags |= D_DIRTY;
 	    return D_USED_CHAR;
 	 }
 	 break;
@@ -1202,6 +1273,7 @@ int d_text_list_proc(int msg, DIALOG *d, int c)
    int listsize, i, a, failure;
    char *selected, *thisitem;
    char *sel = d->dp2;
+   ASSERT(d);
 
    switch (msg) {
 
@@ -1231,17 +1303,17 @@ int d_text_list_proc(int msg, DIALOG *d, int c)
 		  thisitem = (*(getfuncptr)d->dp)(i, NULL);
 		  failure = FALSE;
 
-		  if ((int)d->dp3 < ustrlen(thisitem)) {
-		     for (a=0; a<(int)d->dp3; a++) {
+		  if ((int)((unsigned long)d->dp3) < ustrlen(thisitem)) {
+		     for (a=0; a < (int)((unsigned long)d->dp3); a++) {
 			if (utolower(ugetat(thisitem, a)) != utolower(ugetat(selected, a))) {
 			   failure = TRUE;
 			   break;
 			}
 		     }
 
-		     if ((!failure) && (utolower(ugetat(thisitem, (int)d->dp3)) == utolower(c))) {
+		     if ((!failure) && (utolower(ugetat(thisitem, (int)(unsigned long)d->dp3)) == utolower(c))) {
 			d->d1 = i;
-			d->dp3 = (void *)((int)d->dp3 + 1);
+			d->dp3 = (void *)((unsigned long)d->dp3 + 1);
 
 			if (sel) {
 			   for (i=0; i<listsize; i++)
@@ -1249,9 +1321,7 @@ int d_text_list_proc(int msg, DIALOG *d, int c)
 			}
 
 			_handle_scrollable_scroll(d, listsize, &d->d1, &d->d2);
-			scare_mouse();
-			SEND_MESSAGE(d, MSG_DRAW, 0);
-			unscare_mouse();
+			object_message(d, MSG_DRAW, 0);
 			return D_USED_CHAR;
 		     }
 		  }
@@ -1287,7 +1357,7 @@ void _draw_textbox(char *thetext, int *listsize, int draw, int offset,
    int y1 = y+4;
    int x1;
    int len;
-   int ww = w-4;
+   int ww = w-6;
    char s[16];
    char text[16];
    char space[16];
@@ -1299,6 +1369,7 @@ void _draw_textbox(char *thetext, int *listsize, int draw, int offset,
    int width;
    int line = 0;
    int i = 0;
+   int noignore;
 
    usetc(s+usetc(s, '.'), 0);
    usetc(text+usetc(text, ' '), 0);
@@ -1313,15 +1384,13 @@ void _draw_textbox(char *thetext, int *listsize, int draw, int offset,
    /* do some drawing setup */
    if (draw) {
       /* initial start blanking at the top */
-      rectfill(screen, x+2, y+2, x+w-1, y1-1, deselect);
+      rectfill(screen, x+2, y+2, x+w-3, y1-1, deselect);
    }
 
    /* choose the text color */
-   if (disabled) 
+   if (disabled)
       fg = disable;
 
-   text_mode(deselect);
-
    /* loop over the entire string */
    while (1) {
       width = 0;
@@ -1350,13 +1419,20 @@ void _draw_textbox(char *thetext, int *listsize, int draw, int offset,
 	    if (wword) {
 	       /* remember where we were */
 	       oldscan = scanned;
+	       noignore = FALSE;
 
 	       /* go backwards looking for start of word */
 	       while (!uisspace(ugetc(scanned))) {
 		  /* don't wrap too far */
 		  if (scanned == printed) {
 		     /* the whole line is filled, so stop here */
-		     scanned = oldscan;
+		     tmp = ptmp = scanned;
+		     while (ptmp != oldscan) {
+			ptmp = tmp;
+			tmp += uwidth(tmp);
+		     }
+		     scanned = ptmp;
+		     noignore = TRUE;
 		     break;
 		  }
 		  /* look further backwards to wrap */
@@ -1368,8 +1444,12 @@ void _draw_textbox(char *thetext, int *listsize, int draw, int offset,
 		  scanned = ptmp;
 	       }
 	       /* put the space at the end of the line */
-	       ignore = scanned;
-	       scanned += uwidth(scanned);
+	       if (!noignore) {
+		  ignore = scanned;
+		  scanned += uwidth(scanned);
+	       }
+	       else
+		  ignore = NULL;
 
 	       /* check for endline at the convenient place */
 	       if (ugetc(scanned) == '\n') 
@@ -1385,7 +1465,7 @@ void _draw_textbox(char *thetext, int *listsize, int draw, int offset,
       }
 
       /* check if we are to print it */
-      if ((draw) && (line >= offset) && (y1+text_height(font) < (y+h-1))) {
+      if ((draw) && (line >= offset) && (y1+text_height(font) < (y+h-3))) {
 	 x1 = x+4;
 
 	 /* the initial blank bit */
@@ -1405,16 +1485,16 @@ void _draw_textbox(char *thetext, int *listsize, int draw, int offset,
 	       case '\t':
 		  for (i=0; i<tabsize; i++) {
 		     usetc(s+usetc(s, ' '), 0);
-		     textout(screen, font, s, x1, y1, fg);
+		     textout_ex(screen, font, s, x1, y1, fg, deselect);
 		     x1 += text_length(font, s);
 		  }
 		  break;
 
-	       /* print a normal charater */
+	       /* print a normal character */
 	       default:
 		  if (printed != ignore) {
 		     usetc(s+usetc(s, ugetc(printed)), 0);
-		     textout(screen, font, s, x1, y1, fg);
+		     textout_ex(screen, font, s, x1, y1, fg, deselect);
 		     x1 += text_length(font, s);
 		  }
 	    }
@@ -1423,8 +1503,8 @@ void _draw_textbox(char *thetext, int *listsize, int draw, int offset,
 	    printed += uwidth(printed);
 	 }
 	 /* the last blank bit */
-	 if (x1 < x+w-1) 
-	    rectfill(screen, x1, y1, x+w-1, y1+text_height(font)-1, deselect);
+	 if (x1 <= x+w-3) 
+	    rectfill(screen, x1, y1, x+w-3, y1+text_height(font)-1, deselect);
 
 	 /* print the line end */
 	 y1 += text_height(font);
@@ -1437,14 +1517,15 @@ void _draw_textbox(char *thetext, int *listsize, int draw, int offset,
       /* check if we are at the end of the string */
       if (!ugetc(printed)) {
 	 /* the under blank bit */
-	 if (draw) 
-	    rectfill(screen, x+1, y1, x+w-1, y+h-1, deselect);
+	 if (draw)
+	    rectfill(screen, x+1, y1, x+w-3, y+h-1, deselect);
 
 	 /* tell how many lines we found */
 	 *listsize = line;
 	 return;
       }
    }
+
 }
 
 
@@ -1462,12 +1543,14 @@ void _draw_textbox(char *thetext, int *listsize, int draw, int offset,
 int d_textbox_proc(int msg, DIALOG *d, int c)
 {
    int height, bar, ret = D_O_K;
-   int start, top, bottom,l;
-   int used;
-   int fg_color = (d->flags & D_DISABLED) ? gui_mg_color : d->fg;
+   int start, top, bottom, l;
+   int used, delta;
+   int fg_color;
+   ASSERT(d);
 
+   fg_color = (d->flags & D_DISABLED) ? gui_mg_color : d->fg;
    /* calculate the actual height */
-   height = (d->h-4) / text_height(font);
+   height = (d->h-8) / text_height(font);
 
    switch (msg) {
 
@@ -1501,7 +1584,7 @@ int d_textbox_proc(int msg, DIALOG *d, int c)
 	 /* now do the actual drawing */
 	 _draw_textbox(d->dp, &d->d1, 1, d->d2,
 		       !(d->flags & D_SELECTED), 8,
-		       d->x, d->y, d->w-bar-1, d->h,
+		       d->x, d->y, d->w-bar, d->h,
 		       (d->flags & D_DISABLED),
 		       fg_color, d->bg, gui_mg_color);
 
@@ -1513,13 +1596,13 @@ int d_textbox_proc(int msg, DIALOG *d, int c)
 	 /* figure out if it's on the text or the scrollbar */
 	 bar = (d->d1 > height);
 
-	 if ((!bar) || (gui_mouse_x() < d->x+d->w-12)) {
+	 if ((!bar) || (gui_mouse_x() < d->x+d->w-13)) {
 	    /* clicked on the text area */
 	    ret = D_O_K;
 	 }
 	 else {
 	    /* clicked on the scroll area */
-	    _handle_scrollable_scroll_click(d, d->d1, &d->d2);
+	    _handle_scrollable_scroll_click(d, d->d1, &d->d2, height);
 	 }
 	 break;
 
@@ -1533,7 +1616,7 @@ int d_textbox_proc(int msg, DIALOG *d, int c)
 	    else 
 	       top = 0;
 
-	    l = (d->h-3)/text_height(font);
+	    l = (d->h-8)/text_height(font);
 
 	    bottom = d->d2 + l - 1;
 	    if (bottom >= d->d1-1) 
@@ -1550,9 +1633,9 @@ int d_textbox_proc(int msg, DIALOG *d, int c)
 	    else if ((c>>8) == KEY_END) 
 	       d->d2 = d->d1-l;
 	    else if ((c>>8) == KEY_PGUP) 
-	       d->d2 = d->d2-(bottom-top);
+	       d->d2 -= (bottom-top) ? bottom-top : 1;
 	    else if ((c>>8) == KEY_PGDN) 
-	       d->d2 = d->d2+(bottom-top);
+	       d->d2 += (bottom-top) ? bottom-top : 1;
 	    else 
 	       used = D_O_K;
 
@@ -1566,13 +1649,26 @@ int d_textbox_proc(int msg, DIALOG *d, int c)
 	    used = D_O_K;
 
 	 /* if we changed something, better redraw... */
-	 if (d->d2 != start) {
-	    scare_mouse();
-	    SEND_MESSAGE(d, MSG_DRAW, 0);
-	    unscare_mouse();
-	 }
+	 if (d->d2 != start)
+	    d->flags |= D_DIRTY;
+
 	 ret = used;
 	 break;
+      
+      case MSG_WHEEL:
+	 l = (d->h-8)/text_height(font);
+	 delta = (l > 3) ? 3 : 1;
+
+	 /* scroll, making sure that the list stays in bounds */
+	 start = d->d2;
+	 d->d2 = (c > 0) ? MAX(0, d->d2-delta) : MIN(d->d1-l, d->d2+delta);
+
+	 /* if we changed something, better redraw... */
+	 if (d->d2 != start)
+	    d->flags |= D_DIRTY;
+
+	 ret = D_O_K;
+	 break;
 
       case MSG_WANTFOCUS:
 	 /* if we don't have a scrollbar we can't do anything with the focus */
@@ -1623,6 +1719,7 @@ int d_slider_proc(int msg, DIALOG *d, int c)
    fixed slratio, slmax, slpos;
    int (*proc)(void *cbpointer, int d2value);
    int oldval;
+   ASSERT(d);
 
    /* check for slider direction */
    if (d->h < d->w)
@@ -1773,12 +1870,24 @@ int d_slider_proc(int msg, DIALOG *d, int c)
 		  retval |= (*proc)(d->dp3, d->d2);
 	       }
 
-	       scare_mouse();
-	       SEND_MESSAGE(d, MSG_DRAW, 0);
-	       unscare_mouse();
+	       object_message(d, MSG_DRAW, 0);
 	    }
 	 }
 	 break;
+      
+      case MSG_WHEEL: 
+	 oldval = d->d2;
+	 d->d2 = MID(0, d->d2+c, d->d1);
+	 if (d->d2 != oldval) {
+	    /* call callback function here */
+	    if (d->dp2) {
+	       proc = d->dp2;
+	       retval |= (*proc)(d->dp3, d->d2);
+	    }
+
+	    object_message(d, MSG_DRAW, 0);
+	 }
+	 break;
 
       case MSG_CLICK:
 	 /* track the mouse until it is released */
@@ -1797,7 +1906,7 @@ int d_slider_proc(int msg, DIALOG *d, int c)
 	    if (mp > irange-hh)
 	       mp = irange-hh;
 	    slpos = itofix(mp);
-	    slmax = fdiv(slpos, slratio);
+	    slmax = fixdiv(slpos, slratio);
 	    newpos = fixtoi(slmax);
 	    if (newpos != oldval) {
 	       d->d2 = newpos;
@@ -1808,9 +1917,7 @@ int d_slider_proc(int msg, DIALOG *d, int c)
 		  retval |= (*proc)(d->dp3, d->d2);
 	       }
 
-	       scare_mouse();
-	       SEND_MESSAGE(d, MSG_DRAW, 0);
-	       unscare_mouse();
+	       object_message(d, MSG_DRAW, 0);
 	    }
 
 	    /* let other objects continue to animate */
@@ -1822,3 +1929,20 @@ int d_slider_proc(int msg, DIALOG *d, int c)
    return retval;
 }
 
+
+
+/* Overridable procedures used by standard GUI dialogs.  */
+
+#define MAKE_PROC(proc, default)                                \
+int (*proc)(int, DIALOG *, int);                                \
+int _##proc(int msg, DIALOG *d, int c)                          \
+{                                                               \
+    return proc ? proc(msg, d, c) : default(msg, d, c);         \
+}
+
+MAKE_PROC(gui_shadow_box_proc, d_shadow_box_proc);
+MAKE_PROC(gui_ctext_proc, d_ctext_proc);
+MAKE_PROC(gui_button_proc, d_button_proc);
+MAKE_PROC(gui_edit_proc, d_edit_proc);
+MAKE_PROC(gui_list_proc, d_text_list_proc);
+MAKE_PROC(gui_text_list_proc, d_text_list_proc);
