@@ -271,7 +271,7 @@ static BITMAP *fb_init(int w, int h, int v_w, int v_h, int color_depth)
 
    gfx_fbcon.vid_mem = fix_info.smem_len;
 
-   stride = fix_info.line_length;
+   stride = fix_info.line_length * BYTES_PER_PIXEL(color_depth);
    v_w = my_mode.xres_virtual;
    v_h = my_mode.yres_virtual;
    p = fbaddr;
@@ -407,10 +407,11 @@ static int fb_open_device(void)
       p = getenv("FRAMEBUFFER");
 
       if ((p) && (p[0])) {
-	 _al_sane_strncpy(fname, p, sizeof(fname));
+	 strncpy(fname, p, sizeof(fname)-1);
+	 fname[sizeof(fname)-1] = 0;
       }
       else
-	 _al_sane_strncpy(fname, "/dev/fb0", 1024);
+	 strcpy(fname, "/dev/fb0");
    }
 
    /* open the framebuffer device */
