@@ -34,6 +34,7 @@ static void osx_sys_set_window_title(AL_CONST char *);
 static int osx_sys_set_close_button_callback(void (*proc)(void));
 static int osx_sys_set_display_switch_mode(int mode);
 static void osx_sys_get_gfx_safe_mode(int *driver, struct GFX_MODE *mode);
+static void osx_sys_yield_timeslice(void);
 static int osx_sys_desktop_color_depth(void);
 static int osx_sys_get_desktop_resolution(int *width, int *height);
 
@@ -49,7 +50,6 @@ char osx_window_title[ALLEGRO_MESSAGE_SIZE];
 void (*osx_window_close_hook)(void) = NULL;
 int osx_gfx_mode = OSX_GFX_NONE;
 int osx_emulate_mouse_buttons = FALSE;
-int osx_window_first_expose = FALSE;
 
 
 static RETSIGTYPE (*old_sig_abrt)(int num);
@@ -97,7 +97,7 @@ SYSTEM_DRIVER system_macosx =
    osx_sys_desktop_color_depth,
    osx_sys_get_desktop_resolution,
    osx_sys_get_gfx_safe_mode,
-   _unix_yield_timeslice,
+   osx_sys_yield_timeslice,
    _unix_create_mutex,
    _unix_destroy_mutex,
    _unix_lock_mutex,
@@ -169,8 +169,6 @@ void osx_event_handler()
          case NSKeyDown:
 	    if (_keyboard_installed)
 	       osx_keyboard_handler(TRUE, event);
-	    if (([[event charactersIgnoringModifiers] lossyCString][0] == 'q') && ([event modifierFlags] & NSCommandKeyMask))
-	       [NSApp sendEvent: event];
 	    break;
 	
          case NSKeyUp:
@@ -288,7 +286,7 @@ void osx_event_handler()
             
 	 case NSAppKitDefined:
             switch ([event subtype]) {
-                  
+	       
                case NSApplicationActivatedEventType:
 	          if (osx_window) {
 		     [osx_window invalidateCursorRectsForView: [osx_window contentView]];
@@ -306,13 +304,12 @@ void osx_event_handler()
 	       
 	       case NSWindowMovedEventType:
                   /* This is needed to ensure the shadow gets drawn when the window is
-		   * created. It's weird, but when the window is created on another
-		   * thread, sometimes its shadow doesn't get drawn. The same applies
-		   * to the cursor rectangle, which doesn't seem to be set at window
-		   * creation (it works once you move the mouse though).
+		   * created. It's weird, but sometimes it doesn't get drawn if we omit
+		   * the next two lines... The same applies to the cursor rectangle,
+		   * which doesn't seem to be set at window creation (it works once you
+		   * move the mouse though)
 		   */
-	          if ((osx_window) && (osx_window_first_expose)) {
-		     osx_window_first_expose = FALSE;
+	          if (osx_window) {
                      [osx_window setHasShadow: NO];
                      [osx_window setHasShadow: YES];
 		     [osx_window invalidateCursorRectsForView: [osx_window contentView]];
@@ -507,7 +504,7 @@ static void osx_sys_set_window_title(AL_CONST char *title)
 {
    char tmp[ALLEGRO_MESSAGE_SIZE];
    
-   _al_sane_strncpy(osx_window_title, title, ALLEGRO_MESSAGE_SIZE);
+   strcpy(osx_window_title, title);
    do_uconvert(title, U_CURRENT, tmp, U_UTF8, ALLEGRO_MESSAGE_SIZE);
 
    NSString *ns_title = [NSString stringWithUTF8String: tmp];
@@ -519,8 +516,7 @@ static void osx_sys_set_window_title(AL_CONST char *title)
 
 
 /* osx_sys_set_close_button_callback:
- *  Sets the window close callback. Also used when user hits Command-Q or
- *  selects "Quit" from the application menu.
+ *  Sets the window close callback.
  */
 static int osx_sys_set_close_button_callback(void (*proc)(void))
 {
@@ -555,6 +551,16 @@ static void osx_sys_get_gfx_safe_mode(int *driver, struct GFX_MODE *mode)
 
 
 
+/* osx_sys_yield_timeslice:
+ *  Used to play nice with multitasking.
+ */
+static void osx_sys_yield_timeslice(void)
+{
+   usleep(30000);
+}
+
+
+
 /* osx_sys_desktop_color_depth:
  *  Queries the desktop color depth.
  */
