@@ -8,10 +8,7 @@
  *                                           /\____/
  *                                           \_/__/
  *
- *      Video driver for VGA tweaked modes (aka mode-X). This file lives
- *      in the misc directory because although it currently only works
- *      on DOS, it can potentially be shared by the console mode Linux
- *      version.
+ *      Video driver for VGA tweaked modes (aka mode-X).
  *
  *      By Shawn Hargreaves.
  *
@@ -23,21 +20,26 @@
  */
 
 
+#include <string.h>
+
 #include "allegro.h"
-#include "allegro/aintern.h"
-#include "allegro/aintvga.h"
+
+#ifdef GFX_MODEX
+
+#include "allegro/internal/aintern.h"
+#include "allegro/internal/aintvga.h"
 
 #ifdef ALLEGRO_INTERNAL_HEADER
    #include ALLEGRO_INTERNAL_HEADER
 #endif
 
-#ifdef GFX_MODEX
+#if (!defined ALLEGRO_LINUX) || ((defined ALLEGRO_LINUX_VGA) && ((!defined ALLEGRO_WITH_MODULES) || (defined ALLEGRO_MODULE)))
 
 
 
-void _x_draw_sprite_end();
-void _x_blit_from_memory_end();
-void _x_blit_to_memory_end();
+void _x_draw_sprite_end(void);
+void _x_blit_from_memory_end(void);
+void _x_blit_to_memory_end(void);
 
 
 
@@ -56,6 +58,7 @@ GFX_VTABLE __modex_vtable =
    _x_putpixel,
    _x_vline,
    _x_hline,
+   _x_hline,
    _normal_line,
    _normal_rectfill,
    NULL,
@@ -80,8 +83,10 @@ GFX_VTABLE __modex_vtable =
    _x_blit,
    _x_blit_forward,
    _x_blit_backward,
+   _blit_between_formats,
    _x_masked_blit,
    _x_clear_to_color,
+   _pivot_scaled_sprite_flip,
    _x_draw_sprite_end,
    _x_blit_from_memory_end
 };
@@ -91,7 +96,10 @@ GFX_VTABLE __modex_vtable =
 static BITMAP *modex_init(int w, int h, int v_w, int v_h, int color_depth);
 static void modex_exit(BITMAP *b);
 static int modex_scroll(int x, int y);
+static int request_modex_scroll(int x, int y);
+static int poll_modex_scroll(void);
 static void modex_enable_triple_buffer(void);
+static GFX_MODE_LIST *modex_fetch_mode_list(void);
 
 
 
@@ -114,19 +122,50 @@ GFX_DRIVER gfx_modex =
    NULL,                         /* no drawing mode hook */
    _save_vga_mode,
    _restore_vga_mode,
+   modex_fetch_mode_list,
    0, 0,
    TRUE,
    0, 0,
    0x40000,
-   0
+   0,
+   FALSE
 };
 
 
 
+GFX_MODE modex_gfx_modes[] = {
+   { 256, 200, 8 },
+   { 256, 224, 8 },
+   { 256, 240, 8 },
+   { 256, 256, 8 },
+   { 320, 200, 8 },
+   { 320, 240, 8 },
+   { 320, 350, 8 },
+   { 320, 400, 8 },
+   { 320, 480, 8 },
+   { 320, 600, 8 },
+   { 360, 200, 8 },
+   { 360, 240, 8 },
+   { 360, 270, 8 },
+   { 360, 360, 8 },
+   { 360, 400, 8 },
+   { 360, 480, 8 },
+   { 360, 600, 8 },
+   { 376, 282, 8 },
+   { 376, 308, 8 },
+   { 376, 564, 8 },
+   { 400, 150, 8 },
+   { 400, 300, 8 },
+   { 400, 600, 8 },
+   { 0,   0,   0 }
+};
+
+
 #ifdef GFX_XTENDED
 
 
 static BITMAP *xtended_init(int w, int h, int v_w, int v_h, int color_depth);
+static GFX_MODE_LIST *xtended_fetch_mode_list(void);
 
 
 GFX_DRIVER gfx_xtended =
@@ -146,14 +185,24 @@ GFX_DRIVER gfx_xtended =
    NULL, NULL, NULL, NULL,       /* no hardware cursor */
    NULL,                         /* no drawing mode hook */
    NULL, NULL,                   /* no state saving */
+   xtended_fetch_mode_list,
    640, 400,
    TRUE,
    0, 0,
    0x40000,
-   0
+   0,
+   FALSE
 };
 
 
+
+GFX_MODE xtended_gfx_modes[] = {
+   { 640, 400, 8 },
+   { 0,   0,   0 }
+};
+
+
+
 #endif      /* GFX_XTENDED */
 
 
@@ -586,7 +635,7 @@ static BITMAP *modex_init(int w, int h, int v_w, int v_h, int color_depth)
 
    /* check it is a valid resolution */
    if (color_depth != 8) {
-      ustrcpy(allegro_error, get_config_text("Mode-X only supports 8 bit color"));
+      ustrzcpy(allegro_error, ALLEGRO_ERROR_SIZE, get_config_text("Mode-X only supports 8 bit color"));
       return NULL;
    }
 
@@ -595,7 +644,7 @@ static BITMAP *modex_init(int w, int h, int v_w, int v_h, int color_depth)
       mode++;
 
    if (!mode->regs) {
-      ustrcpy(allegro_error, get_config_text("Not a VGA mode-X resolution"));
+      ustrzcpy(allegro_error, ALLEGRO_ERROR_SIZE, get_config_text("Not a VGA mode-X resolution"));
       return NULL;
    }
 
@@ -604,7 +653,7 @@ static BITMAP *modex_init(int w, int h, int v_w, int v_h, int color_depth)
    v_h = MAX(h, v_h);
 
    if (v_h > 0x40000/v_w) {
-      ustrcpy(allegro_error, get_config_text("Virtual screen size too large"));
+      ustrzcpy(allegro_error, ALLEGRO_ERROR_SIZE, get_config_text("Virtual screen size too large"));
       return NULL;
    }
 
@@ -731,12 +780,12 @@ static BITMAP *xtended_init(int w, int h, int v_w, int v_h, int color_depth)
 
    /* check it is a valid resolution */
    if (color_depth != 8) {
-      ustrcpy(allegro_error, get_config_text("Xtended mode only supports 8 bit color"));
+      ustrzcpy(allegro_error, ALLEGRO_ERROR_SIZE, get_config_text("Xtended mode only supports 8 bit color"));
       return NULL;
    }
 
    if ((w != 640) || (h != 400) || (v_w > 640) || (v_h > 400)) {
-      ustrcpy(allegro_error, get_config_text("Xtended mode only supports 640x400"));
+      ustrzcpy(allegro_error, ALLEGRO_ERROR_SIZE, get_config_text("Xtended mode only supports 640x400"));
       return NULL;
    }
 
@@ -750,7 +799,7 @@ static BITMAP *xtended_init(int w, int h, int v_w, int v_h, int color_depth)
    addr = _set_vga_mode(0x100);
 
    if (!addr) {
-      ustrcpy(allegro_error, get_config_text("VESA mode 0x100 not available"));
+      ustrzcpy(allegro_error, ALLEGRO_ERROR_SIZE, get_config_text("VESA mode 0x100 not available"));
       return NULL;
    }
 
@@ -771,6 +820,26 @@ static BITMAP *xtended_init(int w, int h, int v_w, int v_h, int color_depth)
 }
 
 
+
+/* xtended_fetch_mode_list:
+ *  Creates a list of of currently implemented Xtended modes.
+ */
+static GFX_MODE_LIST *xtended_fetch_mode_list(void)
+{
+   GFX_MODE_LIST *mode_list;
+
+   mode_list = malloc(sizeof(GFX_MODE_LIST));
+   if (!mode_list) return NULL;
+   
+   mode_list->mode = malloc(sizeof(xtended_gfx_modes));
+   if (!mode_list->mode) return NULL;
+
+   memcpy(mode_list->mode, xtended_gfx_modes, sizeof(xtended_gfx_modes));
+   mode_list->num_modes = sizeof(xtended_gfx_modes) / sizeof(GFX_MODE) - 1;
+
+   return mode_list;
+}
+
 #endif      /* GFX_XTENDED */
 
 
@@ -808,13 +877,10 @@ static int modex_scroll(int x, int y)
  *  can be used together with poll_modex_scroll to implement triple buffered
  *  animation systems (see examples/ex20.c).
  */
-int request_modex_scroll(int x, int y)
+static int request_modex_scroll(int x, int y)
 {
    long a = x + (y * VIRTUAL_W);
 
-   if (gfx_driver != &gfx_modex)
-      return -1;
-
    DISABLE();
 
    _vsync_out_h();
@@ -844,7 +910,7 @@ int request_modex_scroll(int x, int y)
  *  set by request_modex_scroll(). Only works if vertical retrace interrupts 
  *  are enabled.
  */
-int poll_modex_scroll()
+static int poll_modex_scroll(void)
 {
    if ((_retrace_hpp_value < 0) || (!_timer_use_retrace))
       return FALSE;
@@ -857,7 +923,7 @@ int poll_modex_scroll()
 /* modex_enable_triple_buffer:
  *  Tries to turn on triple buffering mode, if that is currently possible.
  */
-static void modex_enable_triple_buffer()
+static void modex_enable_triple_buffer(void)
 {
    if ((!_timer_use_retrace) && (timer_can_simulate_retrace()))
       timer_simulate_retrace(TRUE);
@@ -870,40 +936,6 @@ static void modex_enable_triple_buffer()
 
 
 
-/* split_modex_screen:
- *  Enables a horizontal split screen at the specified line.
- *  Based on code from Paul Fenwick's XLIBDJ, which was in turn based 
- *  on Michael Abrash's routines in PC Techniques, June 1991.
- */
-void split_modex_screen(int line)
-{
-   if (gfx_driver != &gfx_modex)
-      return;
-
-   if (line < 0)
-      line = 0;
-   else if (line >= SCREEN_H)
-      line = 0;
-
-   _screen_split_position = line;
-
-   /* adjust the line for double-scanned modes */
-   if (SCREEN_H <= 150)
-      line <<= 2;
-   else if (SCREEN_H <= 300)
-      line <<= 1;
-
-   /* disable panning of the split screen area */
-   _alter_vga_register(0x3C0, 0x30, 0x20, 0x20);
-
-   /* set the line compare registers */
-   _write_vga_register(0x3D4, 0x18, (line-1) & 0xFF);
-   _alter_vga_register(0x3D4, 7, 0x10, ((line-1) & 0x100) >> 4);
-   _alter_vga_register(0x3D4, 9, 0x40, ((line-1) & 0x200) >> 3);
-}
-
-
-
 /* x_write:
  *  Inline helper for the C drawing functions: writes a pixel onto a
  *  mode-X bitmap, performing clipping but ignoring the drawing mode.
@@ -1115,9 +1147,9 @@ void _x_draw_lit_sprite(BITMAP *bmp, BITMAP *sprite, int x, int y, int color)
 /* _x_draw_rle_sprite:
  *  Draws an RLE sprite onto a mode-X screen.
  */
-void _x_draw_rle_sprite(BITMAP *bmp, RLE_SPRITE *sprite, int x, int y)
+void _x_draw_rle_sprite(BITMAP *bmp, AL_CONST RLE_SPRITE *sprite, int x, int y)
 {
-   signed char *p = sprite->dat;
+   AL_CONST signed char *p = sprite->dat;
    int c;
    int x_pos, y_pos;
    int lgap, width;
@@ -1219,9 +1251,9 @@ void _x_draw_rle_sprite(BITMAP *bmp, RLE_SPRITE *sprite, int x, int y)
 /* _x_draw_trans_rle_sprite:
  *  Draws an RLE sprite onto a mode-X screen.
  */
-void _x_draw_trans_rle_sprite(BITMAP *bmp, RLE_SPRITE *sprite, int x, int y)
+void _x_draw_trans_rle_sprite(BITMAP *bmp, AL_CONST RLE_SPRITE *sprite, int x, int y)
 {
-   signed char *p = sprite->dat;
+   AL_CONST signed char *p = sprite->dat;
    int c;
    int x_pos, y_pos;
    int lgap, width;
@@ -1325,9 +1357,9 @@ void _x_draw_trans_rle_sprite(BITMAP *bmp, RLE_SPRITE *sprite, int x, int y)
 /* _x_draw_lit_rle_sprite:
  *  Draws a tinted RLE sprite onto a mode-X screen.
  */
-void _x_draw_lit_rle_sprite(BITMAP *bmp, RLE_SPRITE *sprite, int x, int y, int color)
+void _x_draw_lit_rle_sprite(BITMAP *bmp, AL_CONST RLE_SPRITE *sprite, int x, int y, int color)
 {
-   signed char *p = sprite->dat;
+   AL_CONST signed char *p = sprite->dat;
    int c;
    int x_pos, y_pos;
    int lgap, width;
@@ -1429,7 +1461,7 @@ void _x_draw_lit_rle_sprite(BITMAP *bmp, RLE_SPRITE *sprite, int x, int y, int c
 /* _x_draw_character:
  *  Draws a character from a proportional font onto a mode-X screen.
  */
-void _x_draw_character(BITMAP *bmp, BITMAP *sprite, int x, int y, int color)
+void _x_draw_character(BITMAP *bmp, BITMAP *sprite, int x, int y, int color, int bg)
 {
    int c1, c2;
 
@@ -1440,8 +1472,8 @@ void _x_draw_character(BITMAP *bmp, BITMAP *sprite, int x, int y, int color)
 	 if (sprite->line[c1][c2])
 	    x_write(bmp, x+c2, y+c1, color);
 	 else {
-	    if (_textmode >= 0)
-	       x_write(bmp, x+c2, y+c1, _textmode);
+	    if (bg >= 0)
+	       x_write(bmp, x+c2, y+c1, bg);
 	 }
       }
    }
@@ -1452,10 +1484,10 @@ void _x_draw_character(BITMAP *bmp, BITMAP *sprite, int x, int y, int color)
 /* _x_draw_glyph:
  *  Draws monochrome text onto a mode-X screen.
  */
-void _x_draw_glyph(BITMAP *bmp, FONT_GLYPH *glyph, int x, int y, int color)
+void _x_draw_glyph(BITMAP *bmp, AL_CONST FONT_GLYPH *glyph, int x, int y, int color, int bg)
 {
-   unsigned char *data = glyph->dat;
-   unsigned char *dat;
+   AL_CONST unsigned char *data = glyph->dat;
+   AL_CONST unsigned char *dat;
    unsigned long addr;
    int w = glyph->w;
    int h = glyph->h;
@@ -1520,8 +1552,8 @@ void _x_draw_glyph(BITMAP *bmp, FONT_GLYPH *glyph, int x, int y, int color)
 	 for (;;) {
 	    if (d & k)
 	       bmp_write8(addr, color);
-	    else if (_textmode >= 0)
-	       bmp_write8(addr, _textmode);
+	    else if (bg >= 0)
+	       bmp_write8(addr, bg);
 
 	    i += 4;
 	    if (i >= w)
@@ -1543,4 +1575,95 @@ void _x_draw_glyph(BITMAP *bmp, FONT_GLYPH *glyph, int x, int y, int color)
 
 
 
+/* modex_fetch_mode_list:
+ *  Creates a list of of currently implemented ModeX modes.
+ */
+static GFX_MODE_LIST *modex_fetch_mode_list(void)
+{
+   GFX_MODE_LIST *mode_list;
+
+   mode_list = malloc(sizeof(GFX_MODE_LIST));
+   if (!mode_list) return NULL;
+
+   mode_list->mode = malloc(sizeof(modex_gfx_modes));
+   if (!mode_list->mode) return NULL;
+
+   memcpy(mode_list->mode, modex_gfx_modes, sizeof(modex_gfx_modes));
+   mode_list->num_modes = sizeof(modex_gfx_modes) / sizeof(GFX_MODE) - 1;
+
+   return mode_list;
+}
+
+
+
+/* split_modex_screen:
+ *  Enables a horizontal split screen at the specified line.
+ *  Based on code from Paul Fenwick's XLIBDJ, which was in turn based 
+ *  on Michael Abrash's routines in PC Techniques, June 1991.
+ */
+#ifdef ALLEGRO_MODULE
+static void module_split_modex_screen(int line)
+#else
+void split_modex_screen(int line)
+#endif
+{
+   if (gfx_driver != &gfx_modex)
+      return;
+
+   if (line < 0)
+      line = 0;
+   else if (line >= SCREEN_H)
+      line = 0;
+
+   _screen_split_position = line;
+
+   /* adjust the line for double-scanned modes */
+   if (SCREEN_H <= 150)
+      line <<= 2;
+   else if (SCREEN_H <= 300)
+      line <<= 1;
+
+   /* disable panning of the split screen area */
+   _alter_vga_register(0x3C0, 0x30, 0x20, 0x20);
+
+   /* set the line compare registers */
+   _write_vga_register(0x3D4, 0x18, (line-1) & 0xFF);
+   _alter_vga_register(0x3D4, 7, 0x10, ((line-1) & 0x100) >> 4);
+   _alter_vga_register(0x3D4, 9, 0x40, ((line-1) & 0x200) >> 3);
+}
+
+
+
+#endif      /* (!defined ALLEGRO_LINUX) || ((defined ALLEGRO_LINUX_VGA) && ... */
+
+
+#if (defined ALLEGRO_LINUX_VGA) && (defined ALLEGRO_WITH_MODULES)
+
+void (*_split_modex_screen_ptr)(int);
+
+void (split_modex_screen)(int line)
+{
+   _split_modex_screen_ptr(line);
+}
+
+#endif
+
+
+#if (defined ALLEGRO_LINUX_VGA) && (defined ALLEGRO_MODULE)
+
+/* _module_init:
+ *  Called when loaded as a dynamically linked module.
+ */
+void _module_init_modex(int system_driver)
+{
+   if (system_driver == SYSTEM_LINUX)
+      _unix_register_gfx_driver(GFX_MODEX, &gfx_modex, TRUE, FALSE);
+
+   _split_modex_screen_ptr = module_split_modex_screen;
+}
+
+#endif
+
+
+
 #endif      /* ifdef GFX_MODEX */
