@@ -22,7 +22,8 @@
 
 
 
-#define FLOATSINCOS(x, s, c)  _AL_SINCOS((x) * AL_PI / 128.0, s ,c)
+#define floatcos(x)   ((float)(cos((x) * AL_PI / 128.0)))
+#define floatsin(x)   ((float)(sin((x) * AL_PI / 128.0)))
 
 
 #define EPSILON (0.001)
@@ -94,7 +95,8 @@ void get_x_rotate_quat(QUAT *q, float r)
 {
    ASSERT(q);
 
-   FLOATSINCOS(r/2, q->x, q->w);
+   q->w = floatcos(r / 2);
+   q->x = floatsin(r / 2);
    q->y = 0;
    q->z = 0;
 }
@@ -110,8 +112,9 @@ void get_y_rotate_quat(QUAT *q, float r)
 {
    ASSERT(q);
 
-   FLOATSINCOS(r/2, q->y, q->w);
+   q->w = floatcos(r / 2);
    q->x = 0;
+   q->y = floatsin(r / 2);
    q->z = 0;
 }
 
@@ -126,9 +129,10 @@ void get_z_rotate_quat(QUAT *q, float r)
 {
    ASSERT(q);
 
-   FLOATSINCOS(r/2, q->z, q->w);
+   q->w = floatcos(r / 2);
    q->x = 0;
    q->y = 0;
+   q->z = floatsin(r / 2);
 }
 
 
@@ -150,9 +154,12 @@ void get_rotation_quat(QUAT *q, float x, float y, float z)
 
    ASSERT(q);
 
-   FLOATSINCOS(x/2, sx, cx);
-   FLOATSINCOS(y/2, sy, cy);
-   FLOATSINCOS(z/2, sz, cz);
+   sx = floatsin(x / 2);
+   sy = floatsin(y / 2);
+   sz = floatsin(z / 2);
+   cx = floatcos(x / 2);
+   cy = floatcos(y / 2);
+   cz = floatcos(z / 2);
 
    sysz = sy * sz;
    cycz = cy * cz;
@@ -189,7 +196,8 @@ void get_vector_rotation_quat(QUAT *q, float x, float y, float z, float a)
    y /= l;
    z /= l;
 
-   FLOATSINCOS(a/2, s, q->w);
+   q->w = floatcos(a / 2);
+   s    = floatsin(a / 2);
    q->x = s * x;
    q->y = s * y;
    q->z = s * z;
