@@ -62,13 +62,9 @@ _DRIVER_INFO _system_driver_list[] =
 
 
 
-#ifndef ALLEGRO_MACOSX
-
 /* background function manager */
 struct bg_manager *_unix_bg_man;
 
-#endif
-
 
 
 /* _unix_find_resource:
@@ -143,8 +139,6 @@ int _unix_find_resource(char *dest, AL_CONST char *resource, int size)
 
 
 
-#ifndef ALLEGRO_MACOSX
-
 /* _read_os_type:
  *  Set the os_type variable to something sensible.
  */
@@ -160,7 +154,7 @@ void _read_os_type()
 
       /* fetch OS version and revision */
       tmpstr = malloc(strlen(utsn.release)+1);
-      _al_sane_strncpy(tmpstr, utsn.release, strlen(utsn.release)+1);
+      strcpy(tmpstr, utsn.release);
       tmpstr2 = NULL;
 
       for (pos = 0; pos <= strlen(utsn.release); pos++) {
@@ -212,8 +206,6 @@ void _read_os_type()
    os_multitasking = TRUE;
 }
 
-#endif
-
 
 
 /* _unix_sysdrv_yield_timeslice:
@@ -229,7 +221,7 @@ void _unix_yield_timeslice(void)
 
       struct timeval timeout;
       timeout.tv_sec = 0;
-      timeout.tv_usec = 0;
+      timeout.tv_usec = 1;
       select(0, NULL, NULL, NULL, &timeout);
 
    #endif
@@ -237,8 +229,6 @@ void _unix_yield_timeslice(void)
 
 
 
-#ifndef ALLEGRO_MACOSX
-
 /* _unix_get_executable_name:
  *  Return full path to the current executable.
  */
@@ -260,9 +250,9 @@ void _unix_get_executable_name(char *output, int size)
 	 if (temp) {
 	    buffer = temp;
 
-	    _al_sane_strncpy (buffer, start, end - start);
+	    strncpy (buffer, start, end - start);
 	    *(buffer + (end - start)) = '/';
-	    _al_sane_strncpy (buffer + (end - start) + 1, __crt0_argv[0], end - start + 1 + strlen (__crt0_argv[0]) + 1);
+	    strcpy (buffer + (end - start) + 1, __crt0_argv[0]);
 
 	    if ((stat(buffer, &finfo)==0) && (!S_ISDIR (finfo.st_mode))) {
 	       do_uconvert (buffer, U_ASCII, output, U_CURRENT, size);
@@ -281,5 +271,3 @@ void _unix_get_executable_name(char *output, int size)
    do_uconvert (__crt0_argv[0], U_ASCII, output, U_CURRENT, size);
 }
 
-#endif
-
