@@ -80,75 +80,75 @@ static char *dd_err(long err)
    switch (err) {
 
       case DD_OK:
-         _al_sane_strncpy(err_str, "DD_OK", sizeof(err_str));
+         strcpy(err_str, "DD_OK");
          break;
 
       case DDERR_GENERIC:
-         _al_sane_strncpy(err_str, "DDERR_GENERIC", sizeof(err_str));
+         strcpy(err_str, "DDERR_GENERIC");
          break;
 
      case DDERR_INCOMPATIBLEPRIMARY:
-         _al_sane_strncpy(err_str, "DDERR_INCOMPATIBLEPRIMARY", sizeof(err_str));
+         strcpy(err_str, "DDERR_INCOMPATIBLEPRIMARY");
          break;
 
      case DDERR_INVALIDCAPS:
-         _al_sane_strncpy(err_str, "DDERR_INVALIDCAPS", sizeof(err_str));
+         strcpy(err_str, "DDERR_INVALIDCAPS");
          break;
 
       case DDERR_INVALIDOBJECT:
-         _al_sane_strncpy(err_str, "DDERR_INVALIDOBJECT", sizeof(err_str));
+         strcpy(err_str, "DDERR_INVALIDOBJECT");
          break;
 
       case DDERR_INVALIDPARAMS:
-         _al_sane_strncpy(err_str, "DDERR_INVALIDPARAMS", sizeof(err_str));
+         strcpy(err_str, "DDERR_INVALIDPARAMS");
          break;
 
      case DDERR_INVALIDPIXELFORMAT:
-         _al_sane_strncpy(err_str, "DDERR_INVALIDPIXELFORMAT", sizeof(err_str));
+         strcpy(err_str, "DDERR_INVALIDPIXELFORMAT");
          break;
 
       case DDERR_NOFLIPHW:
-         _al_sane_strncpy(err_str, "DDERR_NOFLIPHW", sizeof(err_str));
+         strcpy(err_str, "DDERR_NOFLIPHW");
          break;
 
       case DDERR_NOTFLIPPABLE:
-         _al_sane_strncpy(err_str, "DDERR_NOTFLIPPABLE", sizeof(err_str));
+         strcpy(err_str, "DDERR_NOTFLIPPABLE");
          break;
 
       case DDERR_OUTOFMEMORY:
-         _al_sane_strncpy(err_str, "DDERR_OUTOFMEMORY", sizeof(err_str));
+         strcpy(err_str, "DDERR_OUTOFMEMORY");
          break;
 
       case DDERR_OUTOFVIDEOMEMORY:
-         _al_sane_strncpy(err_str, "DDERR_OUTOFVIDEOMEMORY", sizeof(err_str));
+         strcpy(err_str, "DDERR_OUTOFVIDEOMEMORY");
          break;
 
      case DDERR_PRIMARYSURFACEALREADYEXISTS:
-         _al_sane_strncpy(err_str, "DDERR_PRIMARYSURFACEALREADYEXISTS", sizeof(err_str));
+         strcpy(err_str, "DDERR_PRIMARYSURFACEALREADYEXISTS");
          break;
 
       case DDERR_SURFACEBUSY:
-         _al_sane_strncpy(err_str, "DDERR_SURFACEBUSY", sizeof(err_str));
+         strcpy(err_str, "DDERR_SURFACEBUSY");
          break;
 
       case DDERR_SURFACELOST:
-         _al_sane_strncpy(err_str, "DDERR_SURFACELOST", sizeof(err_str));
+         strcpy(err_str, "DDERR_SURFACELOST");
          break;
 
      case DDERR_UNSUPPORTED:
-         _al_sane_strncpy(err_str, "DDERR_UNSUPPORTED", sizeof(err_str));
+         strcpy(err_str, "DDERR_UNSUPPORTED");
          break;
 
      case DDERR_UNSUPPORTEDMODE:
-         _al_sane_strncpy(err_str, "DDERR_UNSUPPORTEDMODE", sizeof(err_str));
+         strcpy(err_str, "DDERR_UNSUPPORTEDMODE");
          break;
 
      case DDERR_WASSTILLDRAWING:
-         _al_sane_strncpy(err_str, "DDERR_WASSTILLDRAWING", sizeof(err_str));
+         strcpy(err_str, "DDERR_WASSTILLDRAWING");
          break;
 
       default:
-         _al_sane_strncpy(err_str, "DDERR_UNKNOWN", sizeof(err_str));
+         strcpy(err_str, "DDERR_UNKNOWN");
          break;
    }
 
@@ -168,7 +168,7 @@ static LPDIRECTDRAWSURFACE2 create_directdraw2_surface(int w, int h, LPDDPIXELFO
 {
    DDSURFACEDESC ddsurf_desc;
    LPDIRECTDRAWSURFACE ddsurf1;
-   LPVOID ddsurf2;
+   LPDIRECTDRAWSURFACE2 ddsurf2;
    HRESULT hr;
 
    /* describe surface characteristics */
@@ -240,7 +240,7 @@ static LPDIRECTDRAWSURFACE2 create_directdraw2_surface(int w, int h, LPDDPIXELFO
    }
 
    /* retrieve the DirectDrawSurface2 interface */
-   hr = IDirectDrawSurface_QueryInterface(ddsurf1, &IID_IDirectDrawSurface2, &ddsurf2);
+   hr = IDirectDrawSurface_QueryInterface(ddsurf1, &IID_IDirectDrawSurface2, (LPVOID *)&ddsurf2);
 
    /* there is a bug in the COM part of DirectX 3:
     *  If we release the DirectSurface interface, the actual
@@ -254,7 +254,7 @@ static LPDIRECTDRAWSURFACE2 create_directdraw2_surface(int w, int h, LPDDPIXELFO
       return NULL;
    }
 
-   return (LPDIRECTDRAWSURFACE2)ddsurf2;
+   return ddsurf2;
 }
 
 
