@@ -22,8 +22,8 @@
 
 
 /* exported only for asmlock.s */
-char *wd_dirty_lines = NULL;  /* used in WRITE_BANK() */
-void (*update_window) (RECT *rect) = NULL;  /* window updater */
+char* wd_dirty_lines = NULL;    /* used in WRITE_BANK()       */
+void (*update_window) (RECT* rect) = NULL;  /* window updater */
 
 
 static void gfx_directx_set_palette_win(AL_CONST struct RGB *p, int from, int to, int vsync);
@@ -208,12 +208,9 @@ static void paint_win(RECT *rect)
 /* update_matching_window:
  *  Updates a portion of the window when the color depths match.
  */
-static void update_matching_window(RECT *rect)
+static void update_matching_window(RECT* rect)
 {
-   union {
-     POINT p;
-     RECT r;
-   } dest_rect;
+   RECT dest_rect;
 
    _enter_gfx_critical();
 
@@ -223,19 +220,19 @@ static void update_matching_window(RECT *rect)
    }
 
    if (rect)
-      dest_rect.r = *rect;
+      dest_rect = *rect;
    else {
-      dest_rect.r.left   = 0;
-      dest_rect.r.right  = gfx_directx_win.w;
-      dest_rect.r.top    = 0;
-      dest_rect.r.bottom = gfx_directx_win.h;
+      dest_rect.left   = 0;
+      dest_rect.right  = gfx_directx_win.w;
+      dest_rect.top    = 0;
+      dest_rect.bottom = gfx_directx_win.h;
    }
 
-   ClientToScreen(allegro_wnd, &dest_rect.p);
-   ClientToScreen(allegro_wnd, &dest_rect.p + 1);
+   ClientToScreen(allegro_wnd, (LPPOINT)&dest_rect);
+   ClientToScreen(allegro_wnd, (LPPOINT)&dest_rect + 1);
  
    /* blit offscreen backbuffer to the window */
-   IDirectDrawSurface2_Blt(primary_surface->id, &dest_rect.r,
+   IDirectDrawSurface2_Blt(primary_surface->id, &dest_rect,
                            offscreen_surface->id, rect, 0, NULL);
 
    _exit_gfx_critical();
@@ -310,13 +307,9 @@ static int ddsurf_blit_ex(LPDIRECTDRAWSURFACE2 dest_surf, RECT *dest_rect,
 /* update_colorconv_window:
  *  Updates a portion of the window when the color depths don't match.
  */
-static void update_colorconv_window(RECT *rect)
+static void update_colorconv_window(RECT* rect)
 {
-   RECT src_rect;
-   union {
-     POINT p;
-     RECT r;
-   } dest_rect;
+   RECT src_rect, dest_rect;
    HDC src_dc, dest_dc;
    HRESULT hr;
    int direct;
@@ -346,17 +339,17 @@ static void update_colorconv_window(RECT *rect)
       src_rect.bottom = gfx_directx_win.h;
    }
 
-   dest_rect.r = src_rect;
-   ClientToScreen(allegro_wnd, &dest_rect.p);
-   ClientToScreen(allegro_wnd, &dest_rect.p + 1);
+   dest_rect = src_rect;
+   ClientToScreen(allegro_wnd, (LPPOINT)&dest_rect);
+   ClientToScreen(allegro_wnd, (LPPOINT)&dest_rect + 1);
 
    direct = (direct_updating_mode_on &&
-             is_contained(&dest_rect.r, &working_area) &&
+             is_contained(&dest_rect, &working_area) &&
              GetForegroundWindow() == allegro_wnd);
 
    if (direct) {
       /* blit directly to the primary surface without clipping */
-      ddsurf_blit_ex(primary_surface->id, &dest_rect.r,
+      ddsurf_blit_ex(primary_surface->id, &dest_rect,
                      offscreen_surface->id, &src_rect);
    }
    else {
