@@ -194,11 +194,8 @@ static BOOL CALLBACK DSEnumCallback(LPGUID lpGuid, LPCSTR lpcstrDescription, LPC
    if (lpGuid) {
       driver_guids[num_drivers] = lpGuid;
       driver_names[num_drivers] = malloc(strlen(lpcstrDescription)+1);
-      if(driver_names[num_drivers])
-      {
-         _al_sane_strncpy(driver_names[num_drivers], lpcstrDescription, strlen(lpcstrDescription)+1);
-         num_drivers++;
-      }
+      strcpy(driver_names[num_drivers], lpcstrDescription);
+      num_drivers++;
    }
 
    return (num_drivers < MAX_DRIVERS);
@@ -224,13 +221,6 @@ _DRIVER_INFO *_get_win_digi_driver_list(void)
       hr = DirectSoundEnumerate(DSEnumCallback, NULL);
 
       if (hr == DS_OK) {
-         /* Allegro mixer to DirectSound drivers */
-         for (i=0; i<num_drivers; i++) {
-            driver = _get_dsalmix_driver(driver_names[i], driver_guids[i], i);
-
-            _driver_list_append_driver(&driver_list, driver->id, driver, TRUE);
-         }
-
          /* pure DirectSound drivers */
          for (i=0; i<num_drivers; i++) {
             driver = malloc(sizeof(DIGI_DRIVER));
@@ -240,6 +230,13 @@ _DRIVER_INFO *_get_win_digi_driver_list(void)
 
             _driver_list_append_driver(&driver_list, driver->id, driver, TRUE);
          }
+
+         /* Allegro mixer to DirectSound drivers */
+         for (i=0; i<num_drivers; i++) {
+            driver = _get_dsalmix_driver(driver_names[i], driver_guids[i], i);
+
+            _driver_list_append_driver(&driver_list, driver->id, driver, TRUE);
+         }
       }
 
       /* Allegro mixer to WaveOut drivers */
@@ -286,39 +283,39 @@ static char *ds_err(long err)
    switch (err) {
 
       case DS_OK:
-         _al_sane_strncpy(err_str, "DS_OK", sizeof(err_str));
+         strcpy(err_str, "DS_OK");
          break;
 
       case DSERR_ALLOCATED:
-         _al_sane_strncpy(err_str, "DSERR_ALLOCATED", sizeof(err_str));
+         strcpy(err_str, "DSERR_ALLOCATED");
          break;
 
       case DSERR_BADFORMAT:
-         _al_sane_strncpy(err_str, "DSERR_BADFORMAT", sizeof(err_str));
+         strcpy(err_str, "DSERR_BADFORMAT");
          break;
 
       case DSERR_INVALIDPARAM:
-         _al_sane_strncpy(err_str, "DSERR_INVALIDPARAM", sizeof(err_str));
+         strcpy(err_str, "DSERR_INVALIDPARAM");
          break;
 
       case DSERR_NOAGGREGATION:
-         _al_sane_strncpy(err_str, "DSERR_NOAGGREGATION", sizeof(err_str));
+         strcpy(err_str, "DSERR_NOAGGREGATION");
          break;
 
       case DSERR_OUTOFMEMORY:
-         _al_sane_strncpy(err_str, "DSERR_OUTOFMEMORY", sizeof(err_str));
+         strcpy(err_str, "DSERR_OUTOFMEMORY");
          break;
 
       case DSERR_UNINITIALIZED:
-         _al_sane_strncpy(err_str, "DSERR_UNINITIALIZED", sizeof(err_str));
+         strcpy(err_str, "DSERR_UNINITIALIZED");
          break;
 
       case DSERR_UNSUPPORTED:
-         _al_sane_strncpy(err_str, "DSERR_UNSUPPORTED", sizeof(err_str));
+         strcpy(err_str, "DSERR_UNSUPPORTED");
          break;
 
       default:
-         _al_sane_strncpy(err_str, "DSERR_UNKNOWN", sizeof(err_str));
+         strcpy(err_str, "DSERR_UNKNOWN");
          break;
    }
 
@@ -403,21 +400,20 @@ static int digi_directsound_init(int input, int voices)
       goto Error; 
    }
 
-   if ((dscaps.dwFlags & DSCAPS_PRIMARY16BIT) &&
-       ((_sound_bits >= 16) || (_sound_bits <= 0)))
+   if (dscaps.dwFlags & DSCAPS_PRIMARY16BIT)
       _bits = 16;
    else
       _bits = 8;
 
-   if ((dscaps.dwFlags & DSCAPS_PRIMARYSTEREO) && _sound_stereo)
+   if (dscaps.dwFlags & DSCAPS_PRIMARYSTEREO)
       _stereo = 1;
    else
       _stereo = 0;
 
-   if ((dscaps.dwMaxSecondarySampleRate > _sound_freq) && (_sound_freq > 0))
-      _freq = _sound_freq;
+   if (dscaps.dwMaxSecondarySampleRate > 44000)
+      _freq = 44100;
    else
-      _freq = dscaps.dwMaxSecondarySampleRate;
+      _freq = 22050;
 
    _TRACE("DirectSound caps: %u bits, %s, %uHz\n", _bits, _stereo ? "stereo" : "mono", _freq);
 
@@ -561,19 +557,19 @@ static int digi_directsound_mixer_volume(int volume)
 static LPDIRECTSOUNDBUFFER create_dsound_buffer(int len, int freq, int bits, int stereo, int vol, int pan)
 {
    LPDIRECTSOUNDBUFFER snd_buf;
-   WAVEFORMATEX wf;
+   PCMWAVEFORMAT pcmwf;
    DSBUFFERDESC dsbdesc;
    HRESULT hr;
    int switch_mode;
 
    /* setup wave format structure */
-   memset(&wf, 0, sizeof(WAVEFORMATEX));
-   wf.wFormatTag = WAVE_FORMAT_PCM;
-   wf.nChannels = stereo ? 2 : 1;
-   wf.nSamplesPerSec = freq;
-   wf.wBitsPerSample = bits;
-   wf.nBlockAlign = bits * (stereo ? 2 : 1) / 8;
-   wf.nAvgBytesPerSec = wf.nSamplesPerSec * wf.nBlockAlign;
+   memset(&pcmwf, 0, sizeof(PCMWAVEFORMAT));
+   pcmwf.wf.wFormatTag = WAVE_FORMAT_PCM;
+   pcmwf.wf.nChannels = stereo ? 2 : 1;
+   pcmwf.wf.nSamplesPerSec = freq;
+   pcmwf.wBitsPerSample = bits;
+   pcmwf.wf.nBlockAlign = bits * (stereo ? 2 : 1) / 8;
+   pcmwf.wf.nAvgBytesPerSec = pcmwf.wf.nSamplesPerSec * pcmwf.wf.nBlockAlign;
 
    /* setup DSBUFFERDESC structure */
    memset(&dsbdesc, 0, sizeof(DSBUFFERDESC));
@@ -587,7 +583,7 @@ static LPDIRECTSOUNDBUFFER create_dsound_buffer(int len, int freq, int bits, int
       dsbdesc.dwFlags |= DSBCAPS_GLOBALFOCUS;
 
    dsbdesc.dwBufferBytes = len * (bits / 8) * (stereo ? 2 : 1);
-   dsbdesc.lpwfxFormat = &wf;
+   dsbdesc.lpwfxFormat = (LPWAVEFORMATEX)&pcmwf;
 
    /* create buffer */
    hr = IDirectSound_CreateSoundBuffer(directsound, &dsbdesc, &snd_buf, NULL);
