@@ -80,26 +80,26 @@ static char* dinput_err_str(long err)
    switch (err) {
 
       case DIERR_NOTACQUIRED:
-         _al_sane_strncpy(err_str, "the device is not acquired", sizeof(err_str));
+         strcpy(err_str, "the device is not acquired");
          break;
 
       case DIERR_INPUTLOST:
-         _al_sane_strncpy(err_str, "access to the device was lost", sizeof(err_str));
+         strcpy(err_str, "access to the device was lost");
          break;
 
       case DIERR_INVALIDPARAM:
-         _al_sane_strncpy(err_str, "the device does not have a selected data format", sizeof(err_str));
+         strcpy(err_str, "the device does not have a selected data format");
          break;
 
 #ifdef DIERR_OTHERAPPHASPRIO
       /* this is not a legacy DirectX 3 error code */
       case DIERR_OTHERAPPHASPRIO:
-         _al_sane_strncpy(err_str, "can't acquire the device in background", sizeof(err_str));
+         strcpy(err_str, "can't acquire the device in background");
          break;
 #endif
 
       default:
-         _al_sane_strncpy(err_str, "unknown error", sizeof(err_str));
+         strcpy(err_str, "unknown error");
    }
 
    return err_str;
@@ -139,15 +139,6 @@ static void key_dinput_handle_scancode(unsigned char scancode, int pressed)
          ExitProcess(0);  /* unsafe */
       }
 
-      /* emulate PAUSE scancode sequence */
-      if (scancode == DIK_PAUSE) {
-         int i;
-         _handle_pckey(0xE1);
-         for (i=0; i < 5; i++)
-            _handle_pckey(0);
-         return;
-      }
-
       /* dirty hack to let Allegro for Windows use the DOS/Linux way of handling CapsLock */
       if (((scancode == DIK_CAPITAL) || (scancode == DIK_LSHIFT) || (scancode == DIK_RSHIFT))
           && pressed
