@@ -165,30 +165,30 @@ static char* dinput_err_str(long err)
    switch (err) {
 
       case DIERR_ACQUIRED:
-         _al_sane_strncpy(err_str, "the device is acquired", sizeof(err_str));
+         strcpy(err_str, "the device is acquired");
          break;
 
       case DIERR_NOTACQUIRED:
-         _al_sane_strncpy(err_str, "the device is not acquired", sizeof(err_str));
+         strcpy(err_str, "the device is not acquired");
          break;
 
       case DIERR_INPUTLOST:
-         _al_sane_strncpy(err_str, "access to the device was not granted", sizeof(err_str));
+         strcpy(err_str, "access to the device was not granted");
          break;
 
       case DIERR_INVALIDPARAM:
-         _al_sane_strncpy(err_str, "the device does not have a selected data format", sizeof(err_str));
+         strcpy(err_str, "the device does not have a selected data format");
          break;
 
 #ifdef DIERR_OTHERAPPHASPRIO
       /* this is not a legacy DirectX 3 error code */
       case DIERR_OTHERAPPHASPRIO:
-         _al_sane_strncpy(err_str, "can't acquire the device in background", sizeof(err_str));
+         strcpy(err_str, "can't acquire the device in background");
          break;
 #endif
 
       default:
-         _al_sane_strncpy(err_str, "unknown error", sizeof(err_str));
+         strcpy(err_str, "unknown error");
    }
 
    return err_str;
