@@ -303,15 +303,6 @@ static LRESULT CALLBACK directx_wnd_proc(HWND wnd, UINT message, WPARAM wparam,
             return 0;
          break;
 
-      case WM_SYSCOMMAND:
-         if (wparam == SC_MONITORPOWER || wparam == SC_SCREENSAVE) {
-            if (_screensaver_policy == ALWAYS_DISABLED
-                || (_screensaver_policy == FULLSCREEN_DISABLED
-                    && gfx_driver && !gfx_driver->windowed))
-            return 0;
-         }
-         break;
-
       case WM_INITMENUPOPUP:
          wnd_sysmenu = TRUE;
          mouse_set_sysmenu(TRUE);
@@ -464,10 +455,7 @@ static void wnd_thread_proc(HANDLE setup_event)
  */
 int init_directx_window(void)
 {
-   union {
-     POINT p;
-     RECT r;
-   } win_rect;
+   RECT win_rect;
    HANDLE events[2];
    long result;
 
@@ -487,13 +475,13 @@ int init_directx_window(void)
       allegro_wnd = user_wnd;
 
       /* retrieve the window dimensions */
-      GetWindowRect(allegro_wnd, &win_rect.r);
-      ClientToScreen(allegro_wnd, &win_rect.p);
-      ClientToScreen(allegro_wnd, &win_rect.p + 1);
-      wnd_x = win_rect.r.left;
-      wnd_y = win_rect.r.top;
-      wnd_width = win_rect.r.right - win_rect.r.left;
-      wnd_height = win_rect.r.bottom - win_rect.r.top;
+      GetWindowRect(allegro_wnd, &win_rect);
+      ClientToScreen(allegro_wnd, (LPPOINT)&win_rect);
+      ClientToScreen(allegro_wnd, (LPPOINT)&win_rect + 1);
+      wnd_x = win_rect.left;
+      wnd_y = win_rect.top;
+      wnd_width = win_rect.right - win_rect.left;
+      wnd_height = win_rect.bottom - win_rect.top;
    }
    else {
       /* initializes input module without dedicated thread */
