@@ -140,7 +140,6 @@ int main(int argc, char *argv[])
    int doodle_note = -1;
    unsigned char midi_msg[3];
    int doodle_pitch[12] = { 60, 62, 64, 65, 67, 69, 71, 72, 74, 76, 77, 79 };
-   int white, black, red, green;
    int i;
 
    if (allegro_init() != 0)
@@ -173,14 +172,9 @@ int main(int argc, char *argv[])
 	 return 1;
       }
    }
+
    set_palette(default_palette);
-
-   white = makecol(255, 255, 255);
-   black = makecol(  0,   0,   0);
-   red   = makecol(192,   0,   0);
-   green = makecol(  0, 192,   0);
-
-   clear_to_color(screen, white);
+   clear_to_color(screen, makecol(255, 255, 255));
 
    if (install_sound(digicard, midicard, NULL) != 0) {
       set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
@@ -188,16 +182,16 @@ int main(int argc, char *argv[])
       return 1;
    }
 
-   textprintf_centre_ex(screen, font, SCREEN_W/2, 12, red, -1, "Sound code test program for Allegro " ALLEGRO_VERSION_STR ", " ALLEGRO_PLATFORM_STR);
-   textprintf_centre_ex(screen, font, SCREEN_W/2, 24, red, -1, "By Shawn Hargreaves, " ALLEGRO_DATE_STR);
+   textprintf_centre_ex(screen, font, SCREEN_W/2, 12, palette_color[4], makecol(255, 255, 255), "Sound code test program for Allegro " ALLEGRO_VERSION_STR ", " ALLEGRO_PLATFORM_STR);
+   textprintf_centre_ex(screen, font, SCREEN_W/2, 24, palette_color[4], makecol(255, 255, 255), "By Shawn Hargreaves, " ALLEGRO_DATE_STR);
 
-   textprintf_ex(screen, font, 0, 84,  green, -1, "Digital sound driver: %s", digi_driver->name);
-   textprintf_ex(screen, font, 0, 96,  green, -1, "Description: %s", digi_driver->desc);
-   textprintf_ex(screen, font, 0, 108, green, -1, "Voices: %d", digi_driver->voices);
+   textprintf_ex(screen, font, 0, 84,  palette_color[2], makecol(255, 255, 255), "Digital sound driver: %s", digi_driver->name);
+   textprintf_ex(screen, font, 0, 96,  palette_color[2], makecol(255, 255, 255), "Description: %s", digi_driver->desc);
+   textprintf_ex(screen, font, 0, 108, palette_color[2], makecol(255, 255, 255), "Voices: %d", digi_driver->voices);
 
-   textprintf_ex(screen, font, 0, 132, green, -1, "Midi music driver: %s", midi_driver->name);
-   textprintf_ex(screen, font, 0, 144, green, -1, "Description: %s", midi_driver->desc);
-   textprintf_ex(screen, font, 0, 156, green, -1, "Voices: %d", midi_driver->voices);
+   textprintf_ex(screen, font, 0, 132, palette_color[2], makecol(255, 255, 255), "Midi music driver: %s", midi_driver->name);
+   textprintf_ex(screen, font, 0, 144, palette_color[2], makecol(255, 255, 255), "Description: %s", midi_driver->desc);
+   textprintf_ex(screen, font, 0, 156, palette_color[2], makecol(255, 255, 255), "Voices: %d", midi_driver->voices);
 
    while ((i < argc) && (item_count < 9)) {
       if ((stricmp(get_extension(argv[i]), "voc") == 0) || (stricmp(get_extension(argv[i]), "wav") == 0)) {
@@ -209,28 +203,27 @@ int main(int argc, char *argv[])
 	 is_midi[item_count] = 1;
       }
       else {
-	 textprintf_ex(screen, font, 32, 192+item_count*12, red, -1, "Unknown file type '%s'", argv[i]);
-	 item[item_count] = NULL;
-	 item_count++;
-	 i++;
-	 continue;
+	 set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
+	 allegro_message("Unknown file type '%s'\n", argv[i]);
+	 goto get_out;
       }
 
       if (!item[item_count]) {
-	 textprintf_ex(screen, font, 32, 192+item_count*12, red, -1, "Error reading %s", argv[i]);
-      }
-      else {
-	 textprintf_ex(screen, font, 32, 192+item_count*12, black, -1, "%d: %s", item_count+1, argv[i]);
+	 set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
+	 allegro_message("Error reading %s\n", argv[i]);
+	 goto get_out;
       }
 
+      textprintf_ex(screen, font, 32, 192+item_count*12, palette_color[255], makecol(255, 255, 255), "%d: %s", item_count+1, argv[i]);
+
       item_count++;
       i++;
    }
 
-   textprintf_centre_ex(screen, font, SCREEN_W/2, SCREEN_H-48, red, -1, "Press a number 1-9 to trigger a sample or midi file");
-   textprintf_centre_ex(screen, font, SCREEN_W/2, SCREEN_H-36, red, -1, "v/V changes sfx volume, p/P changes sfx pan, and f/F changes sfx frequency");
-   textprintf_centre_ex(screen, font, SCREEN_W/2, SCREEN_H-24, red, -1, "space pauses/resumes MIDI playback, and the arrow keys seek through the tune");
-   textprintf_centre_ex(screen, font, SCREEN_W/2, SCREEN_H-12, red, -1, "Use the function keys to doodle a tune");
+   textprintf_centre_ex(screen, font, SCREEN_W/2, SCREEN_H-48, palette_color[4], makecol(255, 255, 255), "Press a number 1-9 to trigger a sample or midi file");
+   textprintf_centre_ex(screen, font, SCREEN_W/2, SCREEN_H-36, palette_color[4], makecol(255, 255, 255), "v/V changes sfx volume, p/P changes sfx pan, and f/F changes sfx frequency");
+   textprintf_centre_ex(screen, font, SCREEN_W/2, SCREEN_H-24, palette_color[4], makecol(255, 255, 255), "space pauses/resumes MIDI playback, and the arrow keys seek through the tune");
+   textprintf_centre_ex(screen, font, SCREEN_W/2, SCREEN_H-12, palette_color[4], makecol(255, 255, 255), "Use the function keys to doodle a tune");
 
    k = '1';      /* start sound automatically */
 
@@ -241,14 +234,14 @@ int main(int argc, char *argv[])
 	    vol -= 8;
 	    if (vol < 0)
 	       vol = 0;
-	    set_volume(vol, -1);
+	    set_volume(-1, vol);
 	    break;
 
 	 case 'V':
 	    vol += 8;
 	    if (vol > 255)
 	       vol = 255;
-	    set_volume(vol, -1);
+	    set_volume(-1, vol);
 	    break;
 
 	 case 'p':
@@ -312,14 +305,12 @@ int main(int argc, char *argv[])
 	    }
 	    else if (((k & 0xFF) >= '1') && ((k & 0xFF) < '1'+item_count)) {
 	       k = (k & 0xFF) - '1';
-	       if (item[k]) {
-		  if (is_midi[k]) {
-		     play_midi((MIDI *)item[k], FALSE);
-		     paused = FALSE;
-		  }
-		  else
-		     play_sample((SAMPLE *)item[k], vol, pan, freq, FALSE);
+	       if (is_midi[k]) {
+		  play_midi((MIDI *)item[k], FALSE);
+		  paused = FALSE;
 	       }
+	       else
+		  play_sample((SAMPLE *)item[k], vol, pan, freq, FALSE);
 	    }
 	    else {
 	       k >>= 8;
@@ -349,7 +340,7 @@ int main(int argc, char *argv[])
       old_midi_pos = midi_pos;
 
       sprintf(buf, "        midi pos: %ld    vol: %d    pan: %d    freq: %d        ", midi_pos, vol, pan, freq);
-      textout_centre_ex(screen, font, buf, SCREEN_W/2, SCREEN_H-120, black, white);
+      textout_centre_ex(screen, font, buf, SCREEN_W/2, SCREEN_H-120, palette_color[255], makecol(255, 255, 255));
 
       do {
       } while ((!keypressed()) && (midi_pos == old_midi_pos));
@@ -363,14 +354,11 @@ int main(int argc, char *argv[])
 
    get_out:
 
-   for (i=0; i<item_count; i++) {
-      if (!item[i])
-	 continue;
+   for (i=0; i<item_count; i++)
       if (is_midi[i])
 	 destroy_midi((MIDI *)item[i]);
       else
 	 destroy_sample((SAMPLE *)item[i]);
-   }
 
    return 0;
 }
