@@ -65,9 +65,8 @@ typedef struct DATEDIT_GRABBER_INFO
    int type;
    char *grab_ext;
    char *export_ext;
-   DATAFILE *(*grab)(int type, AL_CONST char *filename, DATAFILE_PROPERTY **prop, int depth);  /* TODO: get rid of 'depth' */
+   void *(*grab)(AL_CONST char *filename, long *size, int x, int y, int w, int h, int depth);
    int (*export)(AL_CONST DATAFILE *dat, AL_CONST char *filename);
-   char *prop_types;
 } DATEDIT_GRABBER_INFO;
 
 
@@ -78,7 +77,6 @@ typedef struct DATEDIT_MENU_INFO
    int (*query)(int popup);
    int flags;
    int key;
-   char *prop_types;
 } DATEDIT_MENU_INFO;
 
 
@@ -115,18 +113,15 @@ int datedit_ask(AL_CONST char *fmt, ...);
 
 char *datedit_pretty_name(AL_CONST char *name, AL_CONST char *ext, int force_ext);
 int datedit_clean_typename(AL_CONST char *type);
-AL_CONST char *datedit_get_property(DATAFILE_PROPERTY **prop, int type);
-int datedit_numprop(DATAFILE_PROPERTY **prop, int type);
-void datedit_insert_property(DATAFILE_PROPERTY **prop, int type, AL_CONST char *value);
 void datedit_set_property(DATAFILE *dat, int type, AL_CONST char *value);
 void datedit_find_character(BITMAP *bmp, int *x, int *y, int *w, int *h);
-DATAFILE *datedit_construct(int type, void *dat, long size, DATAFILE_PROPERTY **prop);
 AL_CONST char *datedit_desc(AL_CONST DATAFILE *dat);
 void datedit_sort_datafile(DATAFILE *dat);
 void datedit_sort_properties(DATAFILE_PROPERTY *prop);
 long datedit_asc2ftime(AL_CONST char *time);
 AL_CONST char *datedit_ftime2asc(long time);
 AL_CONST char *datedit_ftime2asc_int(long time);
+int datedit_numprop(AL_CONST DATAFILE *dat, int type);
 AL_CONST char *datedit_grab_ext(int type);
 AL_CONST char *datedit_export_ext(int type);
 
@@ -168,7 +163,7 @@ typedef struct DATEDIT_GRAB_PARAMETERS {
 DATAFILE *datedit_grabnew(DATAFILE *dat, AL_CONST DATEDIT_GRAB_PARAMETERS *params);
 int datedit_grabreplace(DATAFILE *dat, AL_CONST DATEDIT_GRAB_PARAMETERS *params);
 int datedit_grabupdate(DATAFILE *dat, DATEDIT_GRAB_PARAMETERS *params);
-DATAFILE *datedit_grab(DATAFILE_PROPERTY *prop, AL_CONST DATEDIT_GRAB_PARAMETERS *params);
+DATAFILE *datedit_grab(AL_CONST DATEDIT_GRAB_PARAMETERS *params);
 
 int datedit_update(DATAFILE *dat, AL_CONST char *datafile, int force, int verbose, int *changed);
 
