@@ -540,7 +540,7 @@ static void mem_write_file(PACKFILE *f, int size)
 
 
 /* imports data from a GUS patch file */
-static DATAFILE *grab_patch(int type, AL_CONST char *filename, DATAFILE_PROPERTY **prop, int depth)
+static void *grab_patch(AL_CONST char *filename, long *size, int x, int y, int w, int h, int depth)
 {
    PACKFILE *f;
    long sz = file_size(filename);
@@ -586,7 +586,8 @@ static DATAFILE *grab_patch(int type, AL_CONST char *filename, DATAFILE_PROPERTY
 	 }
       }
 
-      return datedit_construct(type, mem, sz, prop);
+      *size = sz;
+      return mem;
    }
 
    /* have to parse the file to adjust the sample format */
@@ -736,7 +737,8 @@ static DATAFILE *grab_patch(int type, AL_CONST char *filename, DATAFILE_PROPERTY
       return NULL;
    }
 
-   return datedit_construct(type, mem, mem_size, prop);
+   *size = mem_size;
+   return mem;
 }
 
 
@@ -818,11 +820,11 @@ static void add_gus_patches(void)
    params.filename = buf;
    params.name = "default_cfg";
    params.type = DAT_DATA;
-   /* params.x = */
-   /* params.y = */
-   /* params.w = */
-   /* params.h = */
-   /* params.colordepth = */
+   params.x = -1;
+   params.y = -1;
+   params.w = -1;
+   params.h = -1;
+   params.colordepth = -1;
    params.relative = FALSE;  /* required (see above) */
 
    printf("Grabbing %s\n", buf);
@@ -1944,11 +1946,11 @@ static void add_soundfont_patches(void)
    params.filename = buf;
    params.name = "default_cfg";
    params.type = DAT_DATA;
-   /* params.x = */
-   /* params.y = */
-   /* params.w = */
-   /* params.h = */
-   /* params.colordepth = */
+   params.x = -1;
+   params.y = -1;
+   params.w = -1;
+   params.h = -1;
+   params.colordepth = -1;
    params.relative = FALSE;  /* required (see above) */
 
    d = datedit_grabnew(datafile, &params);
