@@ -108,9 +108,9 @@ static int export_sample(AL_CONST DATAFILE *dat, AL_CONST char *filename)
 
 
 /* imports a sample from an external file */
-static DATAFILE *grab_sample(int type, AL_CONST char *filename, DATAFILE_PROPERTY **prop, int depth)
+static void *grab_sample(AL_CONST char *filename, long *size, int x, int y, int w, int h, int depth)
 {
-   return datedit_construct(type, load_sample(filename), 0, prop);
+   return load_sample(filename);
 }
 
 
@@ -120,8 +120,6 @@ static int save_sample_in_datafile(DATAFILE *dat, AL_CONST int *fixed_prop, int
 {
    SAMPLE *spl = (SAMPLE *)dat->dat;
 
-   *allegro_errno = 0;
-
    pack_mputw((spl->stereo) ? -spl->bits : spl->bits, f);
    pack_mputw(spl->freq, f);
    pack_mputl(spl->len, f);
@@ -136,10 +134,8 @@ static int save_sample_in_datafile(DATAFILE *dat, AL_CONST int *fixed_prop, int
       }
    }
 
-   if (*allegro_errno)
-      return FALSE;
-   else
-      return TRUE;
+   /* TODO: return FALSE on failure */
+   return TRUE;
 }
 
 
@@ -173,7 +169,6 @@ DATEDIT_GRABBER_INFO datpatch_grabber =
    "pat",
    "pat",
    NULL,
-   NULL,
    NULL
 };
 
@@ -200,7 +195,6 @@ DATEDIT_GRABBER_INFO datsample_grabber =
    "voc;wav",
    "wav",
    grab_sample,
-   export_sample,
-   NULL
+   export_sample
 };
 
